var jsworkflowns = (function(){

// CommonJS require()

function require(p){
    if ('fs' == p) return {};
    var path = require.resolve(p)
      , mod = require.modules[path];
    if (!mod) throw new Error('failed to require "' + p + '"');
    if (!mod.exports) {
      mod.exports = {};
      mod.call(mod.exports, mod, mod.exports, require.relative(path));
    }
    return mod.exports;
  }

require.modules = {};

require.resolve = function (path){
    var orig = path
      , reg = path + '.js'
      , index = path + '/index.js';
    return require.modules[reg] && reg
      || require.modules[index] && index
      || orig;
  };

require.register = function (path, fn){
    require.modules[path] = fn;
  };

require.relative = function (parent) {
    return function(p){
      if ('.' != p.substr(0, 1)) return require(p);
      
      var path = parent.split('/')
        , segs = p.split('/');
      path.pop();
      
      for (var i = 0; i < segs.length; i++) {
        var seg = segs[i];
        if ('..' == seg) path.pop();
        else if ('.' != seg) path.push(seg);
      }

      return require(path.join('/'));
    };
  };


require.register("jsworkflowns", function(module, exports, require){
/*
 *   jsWorkFlow
 *   Copyright(c) 2012, Yin Mingjun.
 * 
 *  jsWorkFlow's Project
 *      2012.03.06: Create By Yin Mingjun - email: yinmingjuncn@gmail.com
 *   
 *   Copyright 2012,  Yin MingJun - email: yinmingjuncn@gmail.com
 *   Dual licensed under the MIT or GPL Version 2 licenses.
 *   http://jquery.org/license
 *  
 */




//use jso as jsoop's alias
var jso = null;

//browser support 
if (typeof (jsoop) === 'undefined') {
    jso = require('jsoop');
}
else {
    jso = jsoop;
}

//register core namespace, force create new one when exists
jso.registerNamespace('jsWorkFlow');
jso.registerNamespace('jsWorkFlow.Activities');

var jsWorkFlow = jso.ns('jsWorkFlow');

//exports for nodes
exports.rootns = jsWorkFlow;

//////////////////////////////////////////////////////////////////////////////////////////
//
// Low level API
//

function jsWorkFlow$setInterval(handler, interval) {
    var retval = setInterval(handler, interval);
    return retval;
}

function jsWorkFlow$clearInterval(intervalID) {
    clearInterval(intervalID);
}

function jsWorkFlow$setTimeout(handler, delay) {
    var retval = setTimeout(handler, delay);
    return retval;
}

function jsWorkFlow$clearTimeout(timeoutID) {
    clearTimeout(timeoutID);
}


jsWorkFlow.setInterval = jsWorkFlow$setInterval;
jsWorkFlow.clearInterval = jsWorkFlow$clearInterval;
jsWorkFlow.setTimeout = jsWorkFlow$setTimeout;
jsWorkFlow.clearTimeout = jsWorkFlow$clearTimeout;

//////////////////////////////////////////////////////////////////////////////////////////
//ActivityState，表示一个jsWorkFlow活动的运行状态

jsWorkFlow.ActivityState = function jsWorkFlow_ActivityState() {
    throw jso.errorNotImplemented();
};

//系统默认的activity的状态
jsWorkFlow.ActivityState.prototype = {
    none: 0,
    start: 1,
    end: 2,
    error: 3
};

//global 数值
//100以内是系统保留的activity状态值，以外的状态通过类的继承关系来确定
jsWorkFlow.ActivityState.min_value = 100;

jso.registerEnum(jso.setTypeName(jsWorkFlow.ActivityState, 'jsWorkFlow.ActivityState'));


//////////////////////////////////////////////////////////////////////////////////////////
//Events，提供jsWorkFlow中事件的管理
//
//TO 开发者：
//    事件机制是jsWorkFlow的基础，因此将事件的触发、执行和管理集中到Events类中。
//
jsWorkFlow.Events = function jsWorkFlow_Events(owner) {
    this._owner = owner;
    this._events = new jso.EventHandlerList();
};

function jsWorkFlow_Events$dispose() {
    this._owner = null;
    this._events = null;
}

function jsWorkFlow_Events$get_events() {
    return this._events;
}

function jsWorkFlow_Events$addHandler(eventName, handler) {
    this._events.addHandler(eventName, handler);
}

function jsWorkFlow_Events$removeHandler(eventName, handler) {
    this._events.removeHandler(eventName, handler);
}

//触发事件
function jsWorkFlow_Events$raiseEvent(eventName, eventArgs) {
    this._events.raiseEvent(eventName, this._owner, eventArgs);
}

jsWorkFlow.Events.prototype = {
    _owner: null,
    _events: null,
    dispose: jsWorkFlow_Events$dispose,
    //property
    get_events: jsWorkFlow_Events$get_events,
    //method
    addHandler: jsWorkFlow_Events$addHandler,
    removeHandler: jsWorkFlow_Events$removeHandler,
    raiseEvent: jsWorkFlow_Events$raiseEvent
};

jso.registerClass(jso.setTypeName(jsWorkFlow.Events, 'jsWorkFlow.Events'));


//////////////////////////////////////////////////////////////////////////////////////////
//Application，表示一个jsWorkFlow的应用运行环境
//
//TO 开发者：
//    Applcation代表一个独立的运行环境，在概念上对应一个传统的进程，可以在这个观察点上关注APP的状态的变化，
//并获取对应的APP事件通知。
//    每个Application都包含一个Scheduler，用于调度job的运行，而activity本身会作为job的内容被调度到Scheduler中
//执行。
//    dataContext是传递给APP的启动参数，字典形式。必须是plane data object。APP会克隆后加入到applicationContext之中。
//
//
jsWorkFlow.Application = function jsWorkFlow_Application(instance, dataContext) {
    var log = jwf$getLogger();
    log.debug("Application create!");

    //事件属于装配件，需一直存在
    this._events = new jsWorkFlow.Events(this);

    this._instance = instance;
    if (!dataContext) {
        dataContext = {};
    }
    this._dataContext = dataContext;

    //调度器属于APP的固定组成部分，activity只是使用者之一，一直可用
    this._scheduler = new jsWorkFlow.Scheduler();

    this._scheduler_stop_handler = jso.createDelegate(this, this.scheduler_stop_handler);

    //accept scheduler's stop event
    this._scheduler.add_stop(this._scheduler_stop_handler);
};

function jsWorkFlow_Application$get_scheduler() {
    return this._scheduler;
}

function jsWorkFlow_Application$get_globalContext() {
    var ins = jsWorkFlow.GlobalContext.getInstance();
    return ins;
}

function jsWorkFlow_Application$get_appContext() {
    return this._appContext;
}

function jsWorkFlow_Application$get_instance() {
    return this._instance;
}

function jsWorkFlow_Application$get_currentContext() {
    //return stack top
    var contextStack = this._contextStack;
    var context = contextStack[contextStack.length - 1];

    return context;
}

function jsWorkFlow_Application$run(keepRunning) {
    var log = jwf$getLogger();
    log.debug("Application run!");

    //应用的上下文，运行期间可用
    this._appContext = new jsWorkFlow.ApplicationContext();

    //将dataContext放置到appContext之中
    if(this._dataContext) {
        var dst = this._appContext;

        for (var key in this._dataContext) {
            if (!this._dataContext.hasOwnProperty(key)) {
                continue;
            }

            dst.setData(key, this._dataContext[key]);
        }
    }

    //activityContext栈，运行期间可用
    this._contextStack = [];

    //启动scheduler
    this._scheduler.start();

    //触发停止事件
    this._events.raiseEvent('start', jso.EventArgs.Empty);

    //通过instance的execute开始执行
    this._instance.execute(this);

    //如果是自动停止，在调度作业后设置stopPendding标志
    if (!keepRunning) {
        //scheduler would stop when queue is empty
        this._scheduler.stop();
    }
}


function jsWorkFlow_Application$scheduler_stop_handler() {
    //触发停止事件
    this._events.raiseEvent('stop', jso.EventArgs.Empty);

}

function jsWorkFlow_Application$dispose() {
    var log = jwf$getLogger();
    log.debug("Application dispose!");

    this._events.dispose();
    this._instance.dispose();
    this._scheduler.dispose();
    this._appContext.dispose();

    this._events = null;
    this._instance = null;
    this._scheduler = null;
    this._appContext = null;
    this._contextStack = null;
    this._scheduler_stop_handler = null;

}


//ActivityContext栈维护
function jsWorkFlow_Application$pushContextStack(activityContext) {
    //将activityContext入栈
    this._contextStack.push(activityContext);
}

function jsWorkFlow_Application$popContextStack() {
    //将栈顶的activityContext出栈，并返回
    var activityContext = this._contextStack.pop();
    return activityContext;
}

//取跳过指定次序的栈顶的context
//index从0开始
function jsWorkFlow_Application$internalContextAt(index) {
    var contextStack = this._contextStack;
    var activityContext = contextStack[contextStack.length - 1 - index];
    return activityContext;
}

//localContextIndex指定不受可见性控制的top stack context
function jsWorkFlow_Application$getDataItem(key, localContextIndex) {
    //根据key值在activityContext栈中查找value，并返回
    //注意：除顶层外，其他层受可见性控制

    if (typeof (localContextIndex) === "undefined") {
        localContextIndex = 0;
    }

    var dataItem = null;

    //try contextStack
    var contextStack = this._contextStack;

    for (var i = contextStack.length - 1; i >= 0; i--) {
        var context = contextStack[i];

        dataItem = context.getDataItem(key);

        //顶层不受可见性的控制
        if (dataItem && (i >= (contextStack.length - localContextIndex - 1))) {
            return dataItem;
        }

        //其他层受可见性的约束
        if (dataItem && dataItem.isPublic) {
            return dataItem;
        }
    }

    //try appContext
    dataItem = this._appContext.getDataItem(key);

    if (dataItem && dataItem.isPublic) {
        return dataItem;
    }

    //try globalContext
    var globalContext = this.get_globalContext();

    dataItem = globalContext.getDataItem(key);
    if (dataItem && dataItem.isPublic) {
        return dataItem;
    }

    return null;
}

//APP级别的数据维护接口
function jsWorkFlow_Application$getData(key, localContextIndex) {
    var dataItem = this.getDataItem(key, localContextIndex);

    if (dataItem) {
        return dataItem.value;
    }

    return null;
}

//如果不存在已经定义的数据项，数据保存在currentContext之中
function jsWorkFlow_Application$setData(key, value, localContextIndex) {
    var dataItem = this.getDataItem(key, localContextIndex);

    if (dataItem) {
        dataItem.value = value;
        return;
    }

    //找不到，在指定的层上创建key、value
    //var context = this.get_currentContext();
    var context = this.internalContextAt(localContextIndex);
    context.setData(key, value);
}

function jsWorkFlow_Application$add_start(handler) {
    this._events.addHandler('start', handler);
}

function jsWorkFlow_Application$remove_start(handler) {
    this._events.removeHandler('start', handler);
}

function jsWorkFlow_Application$add_stop(handler) {
    this._events.addHandler('stop', handler);
}

function jsWorkFlow_Application$remove_stop(handler) {
    this._events.removeHandler('stop', handler);
}

jsWorkFlow.Application.prototype = {
    _instance: null,
    _dataContext: null,
    _appContext: null,
    _contextStack: null,
    _events: null,
    _scheduler: null,
    _scheduler_stop_handler: null,
    //property
    get_scheduler: jsWorkFlow_Application$get_scheduler,
    get_globalContext: jsWorkFlow_Application$get_globalContext,
    get_appContext: jsWorkFlow_Application$get_appContext,
    get_instance: jsWorkFlow_Application$get_instance,
    get_currentContext: jsWorkFlow_Application$get_currentContext,

    //method
    run: jsWorkFlow_Application$run,
    scheduler_stop_handler: jsWorkFlow_Application$scheduler_stop_handler,
    dispose: jsWorkFlow_Application$dispose,
    //ActivityContext栈维护
    pushContextStack: jsWorkFlow_Application$pushContextStack,
    popContextStack: jsWorkFlow_Application$popContextStack,
    internalContextAt: jsWorkFlow_Application$internalContextAt,
    //APP级别的数据维护接口
    getDataItem: jsWorkFlow_Application$getDataItem,
    getData: jsWorkFlow_Application$getData,
    setData: jsWorkFlow_Application$setData,

    //events
    add_start: jsWorkFlow_Application$add_start,
    remove_start: jsWorkFlow_Application$remove_start,
    add_stop: jsWorkFlow_Application$add_stop,
    remove_stop: jsWorkFlow_Application$remove_stop
};

jso.registerClass(jso.setTypeName(jsWorkFlow.Application, 'jsWorkFlow.Application'));

//////////////////////////////////////////////////////////////////////////////////////////
//Instance，表示一个jsWorkFlow的activity组合的运行载体，主要包含rootActivity
//
//TO 开发者：
//    用来表示activity组装的最终结果，于activity本身并无太大的差别。
//    Instance主要的任务是面向装配的，即如何从文档中解析和加载一个描述的Activity。
//    Instance的执行是初始化ActivityContext的栈的数据结构，并将rootActivity调度执行。
//
jsWorkFlow.Instance = function jsWorkFlow_Instance() {
    var log = jwf$getLogger();
    log.debug("Instance create!");

    this._events = new jsWorkFlow.Events(this);
};

function jsWorkFlow_Instance$dispose() {
    var log = jwf$getLogger();
    log.debug("Instance dispose!");

    if (this._rootActivity) {
        this._rootActivity.dispose();
        this._rootActivity = null;
    }

     this._events.dispose();

      this._events = null;
}

//rootActivity属性
function jsWorkFlow_Instance$get_rootActivity() {
    return this._rootActivity;
}

function jsWorkFlow_Instance$set_rootActivity(value) {
    this._rootActivity = value;
}

//加载一个activity描述，并解析，并放置到rootActivity之中
function jsWorkFlow_Instance$loadFromXML(xml) {
}

//清除rootActivity，并清除相关的加载的数据
function jsWorkFlow_Instance$unload() {
}

//开始在APP中运行
function jsWorkFlow_Instance$execute(application) {

    var log = jwf$getLogger();
    log.debug("Instance execute!");

    //TODO:
    //检查application参数和rootActivity
    
    //Create ActivityExecutor，执行rootActivity
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, this._rootActivity, null);

    //创建delegates
    this._initEventHandler = jso.createDelegate(this, this.initEventHandler);
    this._completeEventHandler = jso.createDelegate(this, this.completeEventHandler);

    //attacth delegate to events
    activityExecutor.add_init(this._initEventHandler);
    activityExecutor.add_complete(this._completeEventHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}


//执行init相关操作
function jsWorkFlow_Instance$doInit(eventArgs) {
    var log = jwf$getLogger();
    log.debug("Instance doInit!");

    //TODO:
    //init instance
}

//执行complete相关操作
function jsWorkFlow_Instance$doComplete(eventArgs) {
    var log = jwf$getLogger();
    log.debug("Instance doComplete!");

    //TODO:
    //clear instance
}

function jsWorkFlow_Instance$initEventHandler(eventArgs) {
    var log = jwf$getLogger();
    log.debug("Instance initEventHandler!");

    //先执行自身init相关逻辑
    this.doInit(eventArgs);

    //触发注册的init事件
    this._events.raiseEvent('init', eventArgs);
}

function jsWorkFlow_Instance$add_init(handler) {
    this._events.addHandler('init', handler);
}

function jsWorkFlow_Instance$remove_init(handler) {
    this._events.removeHandler('init', handler);
}

function jsWorkFlow_Instance$completeEventHandler(eventArgs) {
    var log = jwf$getLogger();
    log.debug("Instance completeEventHandler!");

    //触发注册的complete事件
    this._events.raiseEvent('complete', eventArgs);

    //执行自身complete相关逻辑
    this.doComplete(eventArgs);
}

function jsWorkFlow_Instance$add_complete(handler) {
    this._events.addHandler('complete', handler);
}

function jsWorkFlow_Instance$remove_complete(handler) {
    this._events.removeHandler('complete', handler);
}


jsWorkFlow.Instance.prototype = {
    _rootActivity: null,
    _events: null,
    _initEventHandler: null,
    _completeEventHandler:null,
    dispose: jsWorkFlow_Instance$dispose,
    //property
    get_rootActivity: jsWorkFlow_Instance$get_rootActivity,
    set_rootActivity: jsWorkFlow_Instance$set_rootActivity,
    //method
    loadFromXML: jsWorkFlow_Instance$loadFromXML,
    unload: jsWorkFlow_Instance$unload,
    execute: jsWorkFlow_Instance$execute,
    //执行init相关操作
    doInit: jsWorkFlow_Instance$doInit,
    //执行complete相关操作
    doComplete: jsWorkFlow_Instance$doComplete,
    //events
    //init事件，做activity执行前的准备工作
    initEventHandler: jsWorkFlow_Instance$initEventHandler,
    add_init: jsWorkFlow_Instance$add_init,
    remove_init: jsWorkFlow_Instance$remove_init,
    //complete事件，做activity执行结束后的清理工作
    completeEventHandler: jsWorkFlow_Instance$completeEventHandler,
    add_complete: jsWorkFlow_Instance$add_complete,
    remove_complete: jsWorkFlow_Instance$remove_complete

};

jso.registerClass(jso.setTypeName(jsWorkFlow.Instance, 'jsWorkFlow.Instance'));


//////////////////////////////////////////////////////////////////////////////////////////
//ActivityEventArgs，提供jsWorkFlow的Activity的事件响应参数
//
// TO 开发者：
//    activity相关的事件handler中传递的参数，主要是携带context参数。
//    data是可选的参数，看事件上下文是否需要携带额外的数据
//
jsWorkFlow.ActivityEventArgs = function jsWorkFlow_ActivityEventArgs(context, data) {

    jso.initializeBase(jsWorkFlow.ActivityEventArgs, this, []);

    this._context = context;
    this._data = data;


};

function jsWorkFlow_ActivityEventArgs$dispose() {
    this._context = null;
}

function jsWorkFlow_ActivityEventArgs$get_context() {
    return this._context;
}

function jsWorkFlow_ActivityEventArgs$get_data() {
    return this._data;
}

jsWorkFlow.ActivityEventArgs.prototype = {
    _context: null,
    _data: null,
    dispose: jsWorkFlow_ActivityEventArgs$dispose,
    //property
    get_context: jsWorkFlow_ActivityEventArgs$get_context,
    get_data: jsWorkFlow_ActivityEventArgs$get_data
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityEventArgs, 'jsWorkFlow.ActivityEventArgs'), jso.EventArgs);

//////////////////////////////////////////////////////////////////////////////////////////
//ActivityHelper，提供activity帮助方法
//
// TO 开发者：
//    每个activity都是一个状态机，通过变更状态来驱动事件，通过事件来驱动状态变更。
//    ActivityHelper提供activity的公共方法，$jwf是其简写名称。
//
jsWorkFlow.ActivityHelper = function jsWorkFlow_ActivityHelper() {
    throw jso.errorNotImplemented();
};

jsWorkFlow.ActivityHelper.prototype = {
};

//静态的方法
jsWorkFlow.ActivityHelper.startActivity = function jsWorkFlow_ActivityHelper$startActivity(context) {
    //使activity进入start状态
    context.set_activityState(jsWorkFlow.ActivityState.start);
}

jsWorkFlow.ActivityHelper.endActivity = function jsWorkFlow_ActivityHelper$endActivity(context) {
    //使activity进入end状态
    context.set_activityState(jsWorkFlow.ActivityState.end);
}

//抛出异常
jsWorkFlow.ActivityHelper.raiseError = function jsWorkFlow_ActivityHelper$raiseError(context, exception) {
    var errorInfo = new jsWorkFlow.ActivityErrorInfo(context, exception);
    //使activity进入error状态
    context.set_errorInfo(errorInfo);

    //使activity进入end状态
    context.set_activityState(jsWorkFlow.ActivityState.error);
}

//向parent传递异常，内部使用
jsWorkFlow.ActivityHelper._deliverError = function jsWorkFlow_ActivityHelper$_deliverError(activityContext) {
    if (activityContext.get_activityState() !== jsWorkFlow.ActivityState.error) {
        //???
        return;
    }

    //向上传递错误信息
    var errorInfo = activityContext.get_errorInfo();
    var parentContext = activityContext.get_parentContext();
    if (parentContext) {
        //让parent也进入错误状态
        parentContext.set_errorInfo(errorInfo);
        parentContext.set_activityState(jsWorkFlow.ActivityState.error);
    }

}

//从错误中恢复
jsWorkFlow.ActivityHelper._recoverError = function jsWorkFlow_ActivityHelper$_recoverError(activityContext, activityState) {
    if (activityContext.get_activityState() !== jsWorkFlow.ActivityState.error) {
        //???
        return;
    }

    //清空error信息
    activityContext.set_errorInfo(null);
    //使activitycontext进入到指定的状态
    activityContext.set_activityState(activityState);

}

//获取指定栈向上的栈中的错误信息
jsWorkFlow.ActivityHelper._getErrorInfo = function jsWorkFlow_ActivityHelper$_getErrorInfo(activityContext) {
    var errorInfo = null;
    while (activityContext) {
        if (activityContext.get_activityState() !== jsWorkFlow.ActivityState.error) {
            activityContext = activityContext.get_parentContext();
            continue;
        }

        //获取error信息
        errorInfo = activityContext.get_errorInfo();
        break;
    }

    return errorInfo;
}

//查找Local数据项helper
jsWorkFlow.ActivityHelper.getLocalDataItem = function jsWorkFlow_ActivityHelper$getLocalDataItem(context, key) {
    var retval = context.getDataItem();
    return retval;
}

jsWorkFlow.ActivityHelper.getLocalData = function jsWorkFlow_ActivityHelper$getLocalData(context, key) {

    var retval = context.getData(key);
    return retval;
}

jsWorkFlow.ActivityHelper.setLocalData = function jsWorkFlow_ActivityHelper$setLocalData(context, key, value) {
    context.setData(key, value);
}

//通用查找数据项helper方法(含contextStack)
jsWorkFlow.ActivityHelper.getDataItem = function jsWorkFlow_ActivityHelper$getDataItem(context, key) {

    //检查是否有本地数据
    var retval = jsWorkFlow.ActivityHelper.getLocalDataItem(context, key);
    if (retval) {
        return retval;
    }

    //检查contextStack
    var app = context.get_application();
    retval = app.getDataItem();
    return retval;
}

jsWorkFlow.ActivityHelper.getData = function jsWorkFlow_ActivityHelper$getData(context, key) {
    var dataItem = jsWorkFlow.ActivityHelper.getDataItem(context, key);

    var retval = null;
    if (dataItem) {
        retval = dataItem.value;
    }

    return retval;
}

jsWorkFlow.ActivityHelper.setData = function jsWorkFlow_ActivityHelper$setData(context, key, value) {
    var dataItem = jsWorkFlow.ActivityHelper.getDataItem(context, key);

    if (dataItem) {
        dataItem.value = value;
        return;
    }

    context.setData(key, value);
}


//查找APP数据项helper
jsWorkFlow.ActivityHelper.getAppDataItem = function jsWorkFlow_ActivityHelper$getAppDataItem(context, key) {

    var app = context.get_application();
    var appContext = app.get_appContext();

    var retval = appContext.getDataItem();
    return retval;
}

jsWorkFlow.ActivityHelper.getAppData = function jsWorkFlow_ActivityHelper$getAppData(context, key) {
    var dataItem = jsWorkFlow.ActivityHelper.getAppDataItem(context, key);

    var retval = null;
    if (dataItem) {
        retval = dataItem.value;
    }

    return retval;
}

jsWorkFlow.ActivityHelper.setAppData = function jsWorkFlow_ActivityHelper$setAppData(context, key, value) {
    var app = context.get_application();
    var appContext = app.get_appContext();

    appContext.setData(key, value);
}

//查找Global数据项helper
jsWorkFlow.ActivityHelper.getGlobalDataItem = function jsWorkFlow_ActivityHelper$getGlobalDataItem(context, key) {

    var app = context.get_application();
    var globalContext = app.get_globalContext();

    var retval = globalContext.getDataItem();
    return retval;
}

jsWorkFlow.ActivityHelper.getGlobalData = function jsWorkFlow_ActivityHelper$getGlobalData(context, key) {
    var dataItem = jsWorkFlow.ActivityHelper.getGlobalDataItem(context, key);

    var retval = null;
    if (dataItem) {
        retval = dataItem.value;
    }

    return retval;
}

jsWorkFlow.ActivityHelper.setGlobalData = function jsWorkFlow_ActivityHelper$setGlobalData(context, key, value) {
    var app = context.get_application();
    var globalContext = app.get_globalContext();

    globalContext.setData(key, value);
}

//通过activity的serializeContext来恢复activity
jsWorkFlow.ActivityHelper.loadActivity = function jsWorkFlow_ActivityHelper$loadActivity(serializeContext) {

    var activity = null;

    if (serializeContext) {
        var activityType = jso.getType(serializeContext['_@_activityType']);

        //
        activity = new activityType();
        activity.loadSerializeContext(serializeContext);
    }

    return activity;
}

jsWorkFlow.ActivityHelper.saveActivity = function jsWorkFlow_ActivityHelper$saveActivity(activity) {

    var serializeContext = null;

    if (activity) {
        serializeContext = {};
        activity.saveSerializeContext(serializeContext);
    }

    return serializeContext;
}

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityHelper, 'jsWorkFlow.ActivityHelper'));

//make a shortcut for ActivityHelper
var $jwf = jsWorkFlow.ActivityHelper;

//////////////////////////////////////////////////////////////////////////////////////////
//Activity，表示一个jsWorkFlow活动
//每个activity都是一个状态机，通过变更状态来驱动事件，通过事件来驱动状态变更。
// 
// to 开发者：
//    activity的是一个装配装置，和我们书写一个函数类似，装配是描述一个activity提供的功能。
//activity中的事件也是为这个目标服务，对activity的事件的设置可以看成是一个装配的过程，装配
//好的activity可以组装到多个activity中使用。
//    activity的运行期的数据，存储在activity context之中。
//    Activity类是一个抽象基类，本身只提供对activity公用数据、方法的封装，不能作为activity来运行。
//    activity的序列化通过loadSerializeContext和saveSerializeContext来执行，传入的serializeContext是
//序列化数据的载体，要求是基本的数据类型(Primary Data Type)，每个activity里面存放自身的数据，其中有
//两个特殊的数据项，"_@_activityType"用于存放activity类型的full name，"_@_base"用于存放父类的
//serializeContext。
//
jsWorkFlow.Activity = function jsWorkFlow_Activity() {
    var log = jwf$getLogger();
    log.debug("Activity create!");

    this._events = new jsWorkFlow.Events(this);
};

function jsWorkFlow_Activity$dispose() {
    var log = jwf$getLogger();
    log.debug("Activity dispose!");

    this._events.dispose();
    this._events = null;
}

function jsWorkFlow_Activity$getType() {
    return Object.getType(this);
}

function jsWorkFlow_Activity$get_name() {
    return this._name;
}

function jsWorkFlow_Activity$set_name(value) {
    this._name = value;
}

//activity的恢复
function jsWorkFlow_Activity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("Activity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复name属性
    this.set_name(serializeContext['name']);
}

//activity的序列化
function jsWorkFlow_Activity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("Activity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activity';
    
    //保存name属性
    serializeContext['name'] = this.get_name();
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activity$execute(context) {
    var log = jwf$getLogger();
    log.debug("Activity execute!");

    //context必须是jsWorkFlow.ActivityContext类型

    //使activity进入start状态
    $jwf.startActivity(context);

    //由于通过事件来驱动，所以要在什么地方来设置activity的结束状态
    //通过$jwf.endActivity(context);
}

//通知状态变更，触发自己关心的事件变更
function jsWorkFlow_Activity$notifyStateChanged(context, oldState, curState) {
    var log = jwf$getLogger();
    log.debug("Activity notifyStateChanged!");

    //data中传递oldState和curState
    var data = { oldState: oldState, curState: curState };
    var eventArgs = new jsWorkFlow.ActivityEventArgs(context, data);

    //如果是状态不是从error迁移过来，可以通知事件的变更
    //从error状态恢复(oldState是error，而curState不是error)，简单的将activityContext的状态设置成目标状态，不通知状态的迁移
    if (oldState === jsWorkFlow.ActivityState.error) {
        if (curState !== jsWorkFlow.ActivityState.error) {
            this._events.raiseEvent('errorRecovery', eventArgs);
            log.debug("Activity notifyStateChanged jsWorkFlow.ActivityState.start!");
            return;
        }
    }


    if (curState == jsWorkFlow.ActivityState.start) {
        log.debug("Activity notifyStateChanged jsWorkFlow.ActivityState.start!");

        //状态迁移到start，触发start事件
        this._events.raiseEvent('start', eventArgs);
    }
    else if (curState == jsWorkFlow.ActivityState.end) {
        log.debug("Activity notifyStateChanged jsWorkFlow.ActivityState.end!");

        this._events.raiseEvent('end', eventArgs);
    }
    else if (curState == jsWorkFlow.ActivityState.error) {
        this._events.raiseEvent('error', eventArgs);
    }

    //ignore other state value
}


//events
function jsWorkFlow_Activity$add_start(handler) {
    this._events.addHandler('start', handler);
}

function jsWorkFlow_Activity$remove_start(handler) {
    this._events.removeHandler('start', handler);
}

function jsWorkFlow_Activity$add_end(handler) {
    this._events.addHandler('end', handler);
}

function jsWorkFlow_Activity$remove_end(handler) {
    this._events.removeHandler('end', handler);
}

function jsWorkFlow_Activity$add_error(handler) {
    this._events.addHandler('error', handler);
}

function jsWorkFlow_Activity$remove_error(handler) {
    this._events.removeHandler('error', handler);
}

//errorRecovery 事件
function jsWorkFlow_Activity$add_errorRecovery(handler) {
    this._events.addHandler('errorRecovery', handler);
}

function jsWorkFlow_Activity$remove_errorRecovery(handler) {
    this._events.removeHandler('errorRecovery', handler);
}


jsWorkFlow.Activity.prototype = {
    _name: null,
    _events: null,
    dispose: jsWorkFlow_Activity$dispose,
    //property
    //name，只是一个标识
    get_name: jsWorkFlow_Activity$get_name,
    set_name: jsWorkFlow_Activity$set_name,

    //method
    loadSerializeContext: jsWorkFlow_Activity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activity$saveSerializeContext,
    execute: jsWorkFlow_Activity$execute,
    notifyStateChanged: jsWorkFlow_Activity$notifyStateChanged,
    //event handler
    //基类关注的activity的状态只有start和stop，其它状态由派生类来扩展
    //start 事件
    add_start: jsWorkFlow_Activity$add_start,
    remove_start: jsWorkFlow_Activity$remove_start,
    add_end: jsWorkFlow_Activity$add_end,
    remove_end: jsWorkFlow_Activity$remove_end,
    //error 事件
    add_error: jsWorkFlow_Activity$add_error,
    remove_error: jsWorkFlow_Activity$remove_error,
    //errorRecovery 事件
    add_errorRecovery: jsWorkFlow_Activity$add_errorRecovery,
    remove_errorRecovery: jsWorkFlow_Activity$remove_errorRecovery

};

jso.registerClass(jso.setTypeName(jsWorkFlow.Activity, 'jsWorkFlow.Activity'));

//////////////////////////////////////////////////////////////////////////////////////////
//ContextBase，context的基础类，提供一致的数据管理方法
//
//to 开发者：
//    默认数据私有。
//    可以根据defaultVisibilityIsPublic的true、false改变缺省的可见性
//    提供基础getDataItem、getData、setData和copyFrom方法
//
jsWorkFlow.ContextBase = function jsWorkFlow_ContextBase(defaultVisibilityIsPublic) {
    this._defaultVisibilityIsPublic = !!defaultVisibilityIsPublic;
    this._data = {};
};

function jsWorkFlow_ContextBase$dispose() {
    this._data = null;
}

//默认的可见性
function jsWorkFlow_ContextBase$get_defaultVisibilityIsPublic() {
    return this._defaultVisibilityIsPublic;
}

function jsWorkFlow_ContextBase$clear() {
    this._data = {};
}

//是否包含key
function jsWorkFlow_ContextBase$containKey(key) {
    return this._data.hasOwnProperty(key);
}

//获取&设置context中的数据
function jsWorkFlow_ContextBase$getDataItem(key) {
    var item = null;

    if (this._data.hasOwnProperty(key)) {
        item = this._data[key];
    }

    return item;
}

function jsWorkFlow_ContextBase$getData(key) {

    var retval = null;
    var item = this.getDataItem(key);

    if (item) {
        retval = item.value;
    }

    return retval;
}

//visibilitySwitch是可见性的切换开关，如果为false，表示不切换，如果为true，表示切换到和默认不同的可见性
function jsWorkFlow_ContextBase$setData(key, value, visibilitySwitch) {
    var isPublic = (!!visibilitySwitch) ? !this._defaultVisibilityIsPublic : this._defaultVisibilityIsPublic;

    var item = { key: key,
        value: value,
        isPublic: isPublic
    };

    this._data[key] = item;
}

//从源字典中复制数据
function jsWorkFlow_ContextBase$copyFrom(srcDict, deepClone) {
    //TODO: provide extent function in jsoop
    //jQuery.extend.extend(!!deepClone, this._data, srcDict); 
    if(!srcDict)
        return;

    for (var key in srcDict) {
        if (!srcDict.hasOwnProperty(key)) {
            continue;
        }

        this._data[key] = srcDict[key];
    }
}


jsWorkFlow.ContextBase.prototype = {
    _defaultVisibilityIsPublic: false,
    _data: null,
    dispose: jsWorkFlow_ContextBase$dispose,
    get_defaultVisibilityIsPublic: jsWorkFlow_ContextBase$get_defaultVisibilityIsPublic,
    //method
    //获取&设置context中的数据
    clear: jsWorkFlow_ContextBase$clear,
    containKey: jsWorkFlow_ContextBase$containKey,
    getDataItem: jsWorkFlow_ContextBase$getDataItem,
    getData: jsWorkFlow_ContextBase$getData,
    setData: jsWorkFlow_ContextBase$setData,
    copyFrom: jsWorkFlow_ContextBase$copyFrom
    //property
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ContextBase, 'jsWorkFlow.ContextBase'));


//////////////////////////////////////////////////////////////////////////////////////////
//GlobalContext，表示一个jsWorkFlow活动全局的运行上下文的运行环境，数据的生命周期可以跨instance
//
//to 开发者：
//    GlobalContext提供全局共享的数据，和javascript引擎是相同的生命周期，为所有application共享的数据。
//    默认可见性为public。
jsWorkFlow.GlobalContext = function jsWorkFlow_GlobalContext() {
    jso.initializeBase(jsWorkFlow.GlobalContext, this, [true]);
};

function jsWorkFlow_GlobalContext$dispose() {
    jso.callBaseMethod(jsWorkFlow.GlobalContext, this, 'dispose');
}

jsWorkFlow.GlobalContext.prototype = {
    dispose: jsWorkFlow_GlobalContext$dispose
    //property
};

jso.registerClass(jso.setTypeName(jsWorkFlow.GlobalContext, 'jsWorkFlow.GlobalContext'), jsWorkFlow.ContextBase);

jsWorkFlow.GlobalContext._instance = new jsWorkFlow.GlobalContext();

jsWorkFlow.GlobalContext.getInstance = function jsWorkFlow_GlobalContext$getInstance() {
    return jsWorkFlow.GlobalContext._instance;
};


//////////////////////////////////////////////////////////////////////////////////////////
//ApplicationContext，应用级别的上下文，数据的生命周期可以跨Activity
//
//to 开发者：
//    ApplicationContext表示应用程序的运行上下文环境，用于管理应用级别共享的数据，于ActivityContext不同，
//    默认可见性为public，ApplicationContext中的数据默认是对包含的activity开放的。
//
jsWorkFlow.ApplicationContext = function jsWorkFlow_ApplicationContext() {
    jso.initializeBase(jsWorkFlow.ApplicationContext, this, [true]);
};

function jsWorkFlow_ApplicationContext$dispose() {
    jso.callBaseMethod(jsWorkFlow.ApplicationContext, this, 'dispose');
}

jsWorkFlow.ApplicationContext.prototype = {
    dispose: jsWorkFlow_ApplicationContext$dispose
    //property
    //method
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ApplicationContext, 'jsWorkFlow.ApplicationContext'), jsWorkFlow.ContextBase);

//////////////////////////////////////////////////////////////////////////////////////////
//ActivityErrorInfo，表示一个jsWorkFlow活动的错误信息
//
//to 开发者：
//    ActivityErrorInfo描述Activity运行期异常，存储exception对象，发生异常的执行点，以及异常对应
//的执行路径。
//
jsWorkFlow.ActivityErrorInfo = function jsWorkFlow_ActivityErrorInfo(originalContext, exception) {
    this._originalContext = originalContext;
    this._exception = exception;

}

function jsWorkFlow_ActivityErrorInfo$get_originalContext() {
    return this._originalContext;
}

function jsWorkFlow_ActivityErrorInfo$get_exception() {
    return this._exception;
}


jsWorkFlow.ActivityErrorInfo.prototype = {
    _originalContext: null,
    _exception: null,

    //获取产生异常的源信息
    get_originalContext: jsWorkFlow_ActivityErrorInfo$get_originalContext,
    //获取exception对象
    get_exception: jsWorkFlow_ActivityErrorInfo$get_exception
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityErrorInfo, 'jsWorkFlow.ActivityErrorInfo'));

//////////////////////////////////////////////////////////////////////////////////////////
//ActivityContext，表示一个jsWorkFlow活动的运行上下文,数据依附于所在的活动
//
//to 开发者：
//    ActivityContext是Activity运行数据的载体，会存储Activity的数据和运行状态，以及获取Activity运行开始、结束的通知。
//ActivityContext中的数据通过字典来管理，使用简单的key-value对，在数据的可见性上，默认是私有（不能跨层级访问），也可
//以设置为public（自己包含的那些activity能访问这些数据）。
//    在应用程序的级别通过ActivityContext栈管理ActivityContext。
//    Activity的执行结果放置到ActivityContext的result属性上，可以由上级的执行者来获取。
//
jsWorkFlow.ActivityContext = function jsWorkFlow_ActivityContext(application, activity, executor) {
    jso.initializeBase(jsWorkFlow.ActivityContext, this);

    this._application = application;
    this._executor = executor;
    this._activity = activity;
    this._activityState = jsWorkFlow.ActivityState.none;
    this._args = {};
    this._result = null;

};

function jsWorkFlow_ActivityContext$dispose() {
    this._application = null;
    this._executor = null;
    this._activity = null;
    this._args = null;
    this._result = null;

    jso.callBaseMethod(jsWorkFlow.ActivityContext, this, 'dispose');
}

//获取传递给activity的参数
function jsWorkFlow_ActivityContext$getParam(name) {
    return this._args[name];
}

//如果在ActivityContext中存在
function jsWorkFlow_ActivityContext$isError() {
    return !!this._errorInfo
}

//和activityContext关联的application
function jsWorkFlow_ActivityContext$get_application() {
    return this._application;
}

//和activityContext关联的activity
function jsWorkFlow_ActivityContext$get_activity() {
    return this._activity;
}

//executor
function jsWorkFlow_ActivityContext$get_executor() {
    return this._executor;
}

function jsWorkFlow_ActivityContext$get_parentContext() {
    var parentContext = null;
    var executor = this.get_executor();

    if (executor) {
        parentContext = executor.get_parentContext();
    }

    return parentContext;
}

//获取&设置activity的状态
function jsWorkFlow_ActivityContext$get_activityState() {
    return this._activityState;
}

function jsWorkFlow_ActivityContext$set_activityState(value) {
    var oldState = this._activityState;
    var curState = value;

    //状态相同不触发状态变更(jsWorkFlow.ActivityState.error每次都触发状态变更，允许error状态的叠加)
    if ((oldState == value) && (value !== jsWorkFlow.ActivityState.error)) {
        return;
    }

    this._activityState = value;

    var data = { oldState: oldState, curState: curState };
    var eventArgs = new jsWorkFlow.ActivityEventArgs(this, data);

    //通知activity状态变更，activity得到基类会处理jsWorkFlow.ActivityState.start和jsWorkFlow.ActivityState.end
    this._activity.notifyStateChanged(this, oldState, value);

    //如果activity的当前状态为jsWorkFlow.ActivityState.end，表示活动已经结束，触发activityExecutor的complete事件
    if (value == jsWorkFlow.ActivityState.end) {
        this._executor.raiseCompleteEvent(eventArgs);
    }

    //如果activity的当前状态为jsWorkFlow.ActivityState.error，需activityExecutor激活运行期错误的处理过程
    if (value == jsWorkFlow.ActivityState.error) {
        this._executor.processRuntimeError(eventArgs);
    }

}

//获取&设置activity的返回值
function jsWorkFlow_ActivityContext$get_result() {
    return this._result;
}

function jsWorkFlow_ActivityContext$set_result(value) {
    this._result = value;
}

function jsWorkFlow_ActivityContext$get_errorInfo() {
    return this._errorInfo;
}

function jsWorkFlow_ActivityContext$set_errorInfo(value) {
    this._errorInfo = value;
}



jsWorkFlow.ActivityContext.prototype = {
    _application: null,
    _activity: null,
    _executor: null,
    _activityState: jsWorkFlow.ActivityState.none,
    _args: null,
    _result: null,
    _errorInfo: null,
    dispose: jsWorkFlow_ActivityContext$dispose,
    //method
    //获取传递给activity的参数
    getParam: jsWorkFlow_ActivityContext$getParam,
    //判断是否存在异常
    isError: jsWorkFlow_ActivityContext$isError,
    //property
    //和activityContext关联的application
    get_application: jsWorkFlow_ActivityContext$get_application,
    //和activityContext关联的activity
    get_activity: jsWorkFlow_ActivityContext$get_activity,
    //executor
    get_executor: jsWorkFlow_ActivityContext$get_executor,
    //get_parentContext
    get_parentContext: jsWorkFlow_ActivityContext$get_parentContext,

    //获取&设置activity的状态
    get_activityState: jsWorkFlow_ActivityContext$get_activityState,
    set_activityState: jsWorkFlow_ActivityContext$set_activityState,
    //获取&设置activity的返回值
    get_result: jsWorkFlow_ActivityContext$get_result,
    set_result: jsWorkFlow_ActivityContext$set_result,
    //获取&设置activity的exception值，和error状态结合使用
    get_errorInfo: jsWorkFlow_ActivityContext$get_errorInfo,
    set_errorInfo: jsWorkFlow_ActivityContext$set_errorInfo


};

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityContext, 'jsWorkFlow.ActivityContext'), jsWorkFlow.ContextBase);


//////////////////////////////////////////////////////////////////////////////////////////
//ActivityExecutor，表示一个jsWorkFlow的Activity的执行器，提供activity的执行上下文
//
// TO 开发者：
//    ActivityExecutor表示activity的运行的上下文，包含运行activity所需的所有运行期的
//支撑组件，包括application，当前的activity，以及对activity运行状态的初始化和complete事件。
//ActivityExecutor是对activity运行期的支持类，activity组件的作者使用。
//    ActivityExecutor的事件面向activity的作者，提供了preInit, init, complete, postComplete
//几个事件，事件参数都是ActivityEventArgs类型，携带当前的Activity对应的ActivityContext。
//    事件的触发时机如下：
//    (1) preInit发生在doInit之前，也就是ActivityContext还没进入到contextStack。
//    (2) init发生在doInit之后，执行环境已经准备就绪。
//    (3) complete发生在doComplete之前，执行环境还在。
//    (4) postComplete发生在doComplete之后，执行环境以及完全清除。
//
//    关于异常处理：
//    ActivityExecutor需要处理两种情况，一种是在其本身执行过程中俘获的异常；另外是从其子的
//ActivityContext中传递过来的异常。
//    ActivityExecutor在处理异常的时候，会将异常对象(exception)保存到其ActivityContext的exception
//属性中，并设置ActivityContext的状态为error，完成异常的传递。而状态到error的迁移会触发executor的
//doCatchException的执行，在这个方法里，会判断当前的executor是否存在异常的handler，是否对异常有修复，
//如果修复了，会中止异常的向上的传递；
//
jsWorkFlow.ActivityExecutor = function jsWorkFlow_ActivityExecutor(application, activity, parentContext) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor create!");

    this._application = application;
    this._activity = activity;
    this._activityContext = null;
    this._parentContext = parentContext;
    this._events = new jsWorkFlow.Events(this);
};

function jsWorkFlow_ActivityExecutor$dispose() {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor dispose!");

    this._activityContext.dispose();
    this._events.dispose();

    this._application = null;
    this._activity = null;
    this._activityContext = null;
    this._events = null;
    this._deliverError = true;
}

function jsWorkFlow_ActivityExecutor$get_activityContext() {
    return this._activityContext;
}

function jsWorkFlow_ActivityExecutor$get_parentContext() {
    return this._parentContext;
}

//是否向上提交错误
function jsWorkFlow_ActivityExecutor$get_deliverError() {
    return this._deliverError;
}

function jsWorkFlow_ActivityExecutor$set_deliverError(value) {
    this._deliverError = !!value;
}


function _jwf$ae$run_and_check(activityContext, callback, callbackContext) {

    var activityExecutor = activityContext.get_executor();

    try {
        //执行callback
        callback(callbackContext);
    } catch (e) {
        //运行出现异常
        activityExecutor.doCatchException(e);
    }
} 

//内部使用，job的callback的handler
function jsWorkFlow_ActivityExecutor$doJobCallback(jobItem) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor doJobCallback!");

    //构造lamda上下文
    var activityExecutor = this;
    var activity = this._activity;
    var application = this._application;
    var activityContext = this._activityContext;

    //执行
    jsWorkFlow.ActivityExecutor.run_and_check(this._activityContext, function () {
        var eventArgs = new jsWorkFlow.ActivityEventArgs(activityContext);
        //执行前先触发executor的init事件，用于准备activity的执行环境
        activityExecutor.raiseInitEvent(eventArgs);

        //执行activity
        activity.execute(activityContext);

    }, null);
    

    //进入事件通知的状态。

    //job的执行结束
}

//executor的执行入口点
function jsWorkFlow_ActivityExecutor$execute() {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor execute!");

    var activityExecutor = this;
    var activity = this._activity;
    var application = this._application;
    var activityContext = this._activityContext;

    //创建activity的context
    this._activityContext = new jsWorkFlow.ActivityContext(application, activity, this);

    //将activity的执行做成一个job，放到scheduler中执行。
    var callback = jso.createDelegate(this, this.doJobCallback);

    var scheduler = this._application.get_scheduler();

    //创建job对象
    var job = new jsWorkFlow.Job(callback, null);

    //将job放到scheduler中执行
    scheduler.scheduleJob(job);
}

//执行init相关操作
function jsWorkFlow_ActivityExecutor$doInit(eventArgs) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor doInit!");

    //push activity context
    this._application.pushContextStack(this._activityContext);

    //TODO:
    //    Binding params here!
}

//执行complete相关操作
function jsWorkFlow_ActivityExecutor$doComplete(eventArgs) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor doComplete!");

    //TODO:
    //    put result here!

    //pop activity context
    this._application.popContextStack();

}

function jsWorkFlow_ActivityExecutor$doCatchException(exception) {
    var log = jwf$getLogger();
    log.info("ActivityExecutor doCatchException!", exception);
    //设置context的exception
    //推送context进入error状态

    var activityContext = this._activityContext;

    //使activity进入error状态
    $jwf.raiseError(activityContext, exception);
}

//当activityState变成error的时候会被调用
//processRuntimeError，处理在activityContext之间的异常的传递
function jsWorkFlow_ActivityExecutor$processRuntimeError(eventArgs) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor processRuntimeError!");

    var activityContext = this._activityContext;
    var data = eventArgs.get_data();
    var oldState = data.oldState;
    var curState = data.curState;

    //检查当前的executor上是否存在error Handler，如果有，调用其执行
    //如果没有错误的恢复处理，将error状态向parentContext传递
    //如果有errorHandler，errorhandler会通过

    //触发注册的error事件，如果有error的handler，会在error期间将activity的状态变更成非error的状态
    this._events.raiseEvent('error', eventArgs);

    //如果还是错误状态，根据executor的deliverError决定是否将错误状态从当前的activity向上传递
    if (activityContext.get_activityState() === jsWorkFlow.ActivityState.error) {
        //除了deliverError标志，另外还需要检查oldState标志，如果是error向error状态迁移，
        //说明在executor的error处理上下文中又产生了一次error，这种情况下也需要向上传递。
        if (this.get_deliverError() || (oldState === jsWorkFlow.ActivityState.error)) {
            //向上传递错误信息
            $jwf._deliverError(activityContext);
        }
    }

    this._events.raiseEvent('errorComplete', eventArgs);


}

function jsWorkFlow_ActivityExecutor$doErrorRecovery(activityState) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor doErrorRecovery!");
    //修正activityContext中的异常，并使activityContext进入指定的状态
    var activityContext = this._activityContext;
    if (activityContext.get_activityState() !== jsWorkFlow.ActivityState.error) {
        //重复错误恢复？
        log.error("activityState is not [jsWorkFlow.ActivityState.error]!");
        
        return;
    }

    $jwf._recoverError(activityContext, activityState);
}

//触发init相关事件
function jsWorkFlow_ActivityExecutor$raiseInitEvent(eventArgs) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor raiseInitEvent!");

    //触发注册的preInit事件
    this._events.raiseEvent('preInit', eventArgs);

    //先执行自身init相关逻辑
    this.doInit(eventArgs);

    //触发注册的init事件
    this._events.raiseEvent('init', eventArgs);
}

//触发complete相关事件
function jsWorkFlow_ActivityExecutor$raiseCompleteEvent(eventArgs) {
    var log = jwf$getLogger();
    log.debug("ActivityExecutor raiseCompleteEvent!");

    //触发注册的complete事件
    this._events.raiseEvent('complete', eventArgs);

    //执行自身complete相关逻辑
    this.doComplete(eventArgs);

    //触发注册的postComplete事件
    this._events.raiseEvent('postComplete', eventArgs);
}


function jsWorkFlow_ActivityExecutor$add_preInit(handler) {
    this._events.addHandler('preInit', handler);
}

function jsWorkFlow_ActivityExecutor$remove_preInit(handler) {
    this._events.removeHandler('preInit', handler);
}

function jsWorkFlow_ActivityExecutor$add_init(handler) {
    this._events.addHandler('init', handler);
}

function jsWorkFlow_ActivityExecutor$remove_init(handler) {
    this._events.removeHandler('init', handler);
}

function jsWorkFlow_ActivityExecutor$add_complete(handler) {
    this._events.addHandler('complete', handler);
}

function jsWorkFlow_ActivityExecutor$remove_complete(handler) {
    this._events.removeHandler('complete', handler);
}

function jsWorkFlow_ActivityExecutor$add_postComplete(handler) {
    this._events.addHandler('postComplete', handler);
}

function jsWorkFlow_ActivityExecutor$remove_postComplete(handler) {
    this._events.removeHandler('postComplete', handler);
}

function jsWorkFlow_ActivityExecutor$add_error(handler) {
    this._events.addHandler('error', handler);
}

function jsWorkFlow_ActivityExecutor$remove_error(handler) {
    this._events.removeHandler('error', handler);
}


//errorComplete事件，做activity运行期异常的修正
function jsWorkFlow_ActivityExecutor$add_errorComplete(handler) {
    this._events.addHandler('errorComplete', handler);
}

function jsWorkFlow_ActivityExecutor$remove_errorComplete(handler) {
    this._events.removeHandler('errorComplete', handler);
}

jsWorkFlow.ActivityExecutor.prototype = {
    _application: null,
    _activity: null,
    _activityContext: null,
    _parentContext: null,
    _events: null,
    _deliverError: true,
    //应该包含activity，提供activity的执行的上下文环境
    dispose: jsWorkFlow_ActivityExecutor$dispose,

    //property
    get_activityContext: jsWorkFlow_ActivityExecutor$get_activityContext,
    get_parentContext: jsWorkFlow_ActivityExecutor$get_parentContext,
    //是否向上提交错误
    get_deliverError: jsWorkFlow_ActivityExecutor$get_deliverError,
    set_deliverError: jsWorkFlow_ActivityExecutor$set_deliverError,
    //method
    //内部使用，job的callback的handler
    doJobCallback: jsWorkFlow_ActivityExecutor$doJobCallback,
    //executor的执行入口点
    execute: jsWorkFlow_ActivityExecutor$execute,
    //执行init相关操作
    doInit: jsWorkFlow_ActivityExecutor$doInit,
    //执行complete相关操作
    doComplete: jsWorkFlow_ActivityExecutor$doComplete,
    //俘获运行期的异常
    doCatchException: jsWorkFlow_ActivityExecutor$doCatchException,
    //处理运行期的错误
    processRuntimeError: jsWorkFlow_ActivityExecutor$processRuntimeError,
    //从错误状态中恢复
    doErrorRecovery: jsWorkFlow_ActivityExecutor$doErrorRecovery,
    //events
    //事件触发
    raiseInitEvent: jsWorkFlow_ActivityExecutor$raiseInitEvent,
    raiseCompleteEvent: jsWorkFlow_ActivityExecutor$raiseCompleteEvent,
    //preInit事件，做activity执行前的准备工作
    add_preInit: jsWorkFlow_ActivityExecutor$add_preInit,
    remove_preInit: jsWorkFlow_ActivityExecutor$remove_preInit,
    //init事件，做activity执行前的准备工作
    add_init: jsWorkFlow_ActivityExecutor$add_init,
    remove_init: jsWorkFlow_ActivityExecutor$remove_init,
    //complete事件，做activity执行结束后的清理工作
    add_complete: jsWorkFlow_ActivityExecutor$add_complete,
    remove_complete: jsWorkFlow_ActivityExecutor$remove_complete,
    //postComplete事件，做activity执行结束后的清理工作
    add_postComplete: jsWorkFlow_ActivityExecutor$add_postComplete,
    remove_postComplete: jsWorkFlow_ActivityExecutor$remove_postComplete,
    //error事件，做activity运行期异常的修正
    add_error: jsWorkFlow_ActivityExecutor$add_error,
    remove_error: jsWorkFlow_ActivityExecutor$remove_error,
    //errorComplete事件，做activity运行期异常的修正
    add_errorComplete: jsWorkFlow_ActivityExecutor$add_errorComplete,
    remove_errorComplete: jsWorkFlow_ActivityExecutor$remove_errorComplete
};

//class method
jsWorkFlow.ActivityExecutor.run_and_check = _jwf$ae$run_and_check;

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityExecutor, 'jsWorkFlow.ActivityExecutor'));

//////////////////////////////////////////////////////////////////////////////////////////
//Job，提供jsWorkFlow的Scheduler的执行项目
//
//to 开发者：
//    Job代表一个需要运行调度的作业，描述执行体包含的内容。
//    其构造参数包含callback和context，表示需要执行的方法和传递给方法的参数。
jsWorkFlow.Job = function jsWorkFlow_Job(callback, context) {

    this._callback = callback;
    this._context = context;
};

function jsWorkFlow_Job$dispose() {
}

function jsWorkFlow_Job$get_context() {
    return this._context;
}

function jsWorkFlow_Job$get_result() {
    return this._result;
}

function jsWorkFlow_Job$set_result(value) {
    this._result = value;
}

function jsWorkFlow_Job$execute() {
    //执行工作项目
    if (this._callback) {
        this._callback(this);
    }
}

jsWorkFlow.Job.prototype = {
    //work item中应该包含ActivityExecutor，Job通过它来执行activity
    _callback: null,
    _context: null,
    _result: null,
    dispose: jsWorkFlow_Job$dispose,
    get_context: jsWorkFlow_Job$get_context,
    get_result: jsWorkFlow_Job$get_result,
    set_result: jsWorkFlow_Job$set_result,
    execute: jsWorkFlow_Job$execute
    //property
};

jso.registerClass(jso.setTypeName(jsWorkFlow.Job, 'jsWorkFlow.Job'));

//////////////////////////////////////////////////////////////////////////////////////////
//Scheduler，提供jsWorkFlow的Instance的调度器
//
//to 开发者：
//    Scheduler是workflow的任务的执行引擎，驱动job的执行。
//    Scheduler在application中启动和初始化，可以通过事件关注其状态的变化，可以在外部控制其启动、停止、暂停等。
jsWorkFlow.Scheduler = function jsWorkFlow_Scheduler() {
    var log = jwf$getLogger();
    log.debug("Scheduler create!");

    this._events = new jsWorkFlow.Events(this);
};

//const
//调度每10 ms执行一次
jsWorkFlow.Scheduler.interval_timer = 10;
jsWorkFlow.Scheduler.interval_time_bar = 100;

function jsWorkFlow_Scheduler$get_isRunning() {
    return this._isRunning;
}

function jsWorkFlow_Scheduler$get_isStopPending() {
    return this._isStopPending;
}

function jsWorkFlow_Scheduler$get_isPausing() {
    return this._isPausing;
}

function jsWorkFlow_Scheduler$dispose() {
    var log = jwf$getLogger();
    log.debug("Scheduler dispose!");

    if (this._isRunning) {
        this.stop(true);
    }

    this._events.dispose();
    this._events = null;
}

//将job放到运行队列
function jsWorkFlow_Scheduler$scheduleJob(job) {
    //Job必须是jsWorkFlow.Job类型

//    //运行状态（含pausing）下可以将job加入到执行队列
//    if (!this._isRunning || this._isStopPending) {
//        throw jso.errorInvalidOperation("Scheduler not running!");
//    }

    jso.arrayAdd(this._jobQueue, job);
}

//run job的时间片的执行函数
function jsWorkFlow_Scheduler$doExecJobInterval() {
    if (!this._isRunning) {
        return;
    }

    if (this._isPausing) {
        return;
    }

    var jobQueue = this._jobQueue;

    var startTime = (new Date()).getTime();

    for (var i = 0, ilen = jobQueue.length; i < ilen; i++) {

        var job = jobQueue.shift();

        try {
            //执行job
            job.execute();

            //job已经执行完，释放
            job.dispose();
        } catch (e) {
            var log = jwf$getLogger();
            log.error("Scheduler catch exception in doExecJobInterval!", e);
        }

        //检查运行时间
        var curTime = (new Date()).getTime();

        if ((curTime - startTime) > jsWorkFlow.Scheduler.interval_time_bar) {
            //运行时间过长了！需要终止运行，给其他操作让出CPU资源。
            break;
        }
    }

    //如果是stopPending，并且队列执行完闭，升级状态为stop
    if (this._isStopPending) {
        if (jobQueue.length == 0) {
            this.stop(true);
        }
    }

}

//默认start直接进入运行状态，如果isPausing为true，表示启动后处于暂停状态
function jsWorkFlow_Scheduler$start(isPausing) {
    var log = jwf$getLogger();
    log.debug("Scheduler start!");

    if (this._isRunning) {
        throw jso.errorInvalidOperation("Scheduler is already running!");
    }

    //start schedule engine
    var handler = jso.createDelegate(this, this.doExecJobInterval);
    this._intervalID = jsWorkFlow.setInterval(handler, jsWorkFlow.Scheduler.interval_timer);

    this._isRunning = true;
    this._isStopPending = false;
    this._isPausing = !!isPausing;
    this._jobQueue = [];

    this._events.raiseEvent('start', jso.EventArgs.Empty);

}

function jsWorkFlow_Scheduler$pause() {
    var log = jwf$getLogger();
    log.debug("Scheduler pause!");

    //如果没运行，或正在停止，不允许设置为暂停
    if (!this._isRunning || this._isStopPending) {
        throw jso.errorInvalidOperation("Scheduler is not running!");
        return;
    }

    this._isPausing = true;

    this._events.raiseEvent('pause', jso.EventArgs.Empty);
}

function jsWorkFlow_Scheduler$resume() {
    var log = jwf$getLogger();
    log.debug("Scheduler resume!");

    if (!this._isRunning || this._isStopPending) {
        throw jso.errorInvalidOperation("Scheduler is not running!");
    }

    this._isPausing = false;
    this._events.raiseEvent('resume', jso.EventArgs.Empty);
}

//forceStopNow参数如果为true，表示是强制停止，不管是否有正在运行的任务。
function jsWorkFlow_Scheduler$stop(forceStopNow) {
    var log = jwf$getLogger();
    log.debug("Scheduler stop!");

    if (!this._isRunning) {
        throw jso.errorInvalidOperation("Scheduler is not running!");
        return;
    }

    if (!forceStopNow) {
        //对于暂停的状态，如果通过stoppending的方式停止，会进入假死的状态。
        if (this._isPausing) {
            throw jso.errorInvalidOperation("Scheduler is pausing, can't put into stoppending state! please resume it first.");
            return;
        }

        this._isStopPending = true;
        return;
    }

    //forceStop
    this.forceStop();

}

function jsWorkFlow_Scheduler$forceStop() {
    var log = jwf$getLogger();
    log.debug("Scheduler forceStop!");
    //forceStop
    //clean all jobs.
    var jobQueue = this._jobQueue;

    for (var i = 0, ilen = jobQueue.length; i < ilen; i++) {

        var job = jobQueue.shift();

        try {
            //释放job
            job.dispose();
        } catch (e) {
            log.error("app catch exception in forceStop!", e);
        }
    }

    //设置运行状态为stop
    this._isRunning = false;

    //stop scheduler engine
    jsWorkFlow.clearInterval(this._intervalID);
    this._intervalID = -1;

    this._events.raiseEvent('stop', jso.EventArgs.Empty);

}

//events
function jsWorkFlow_Scheduler$add_start(handler) {
    this._events.addHandler('start', handler);
}

function jsWorkFlow_Scheduler$remove_start(handler) {
    this._events.removeHandler('start', handler);
}

function jsWorkFlow_Scheduler$add_pause(handler) {
    this._events.addHandler('pause', handler);
}

function jsWorkFlow_Scheduler$remove_pause(handler) {
    this._events.removeHandler('pause', handler);
}

function jsWorkFlow_Scheduler$add_resume(handler) {
    this._events.addHandler('resume', handler);
}

function jsWorkFlow_Scheduler$remove_resume(handler) {
    this._events.removeHandler('resume', handler);
}

function jsWorkFlow_Scheduler$add_stop(handler) {
    this._events.addHandler('stop', handler);
}

function jsWorkFlow_Scheduler$remove_stop(handler) {
    this._events.removeHandler('stop', handler);
}

jsWorkFlow.Scheduler.prototype = {
    _events: null,
    _isRunning: false,
    _isStopPending: false,
    _isPausing: false,
    _jobQueue: null,
    _intervalID: -1,
    //property
    get_isRunning: jsWorkFlow_Scheduler$get_isRunning,
    get_isStopPending: jsWorkFlow_Scheduler$get_isStopPending,
    get_isPausing: jsWorkFlow_Scheduler$get_isPausing,
    //method
    dispose: jsWorkFlow_Scheduler$dispose,
    scheduleJob: jsWorkFlow_Scheduler$scheduleJob,
    //内部执行
    doExecJobInterval: jsWorkFlow_Scheduler$doExecJobInterval,
    //运行状态控制
    start: jsWorkFlow_Scheduler$start,
    pause: jsWorkFlow_Scheduler$pause,
    resume: jsWorkFlow_Scheduler$resume,
    stop: jsWorkFlow_Scheduler$stop,
    forceStop: jsWorkFlow_Scheduler$forceStop,
    //event
    add_start: jsWorkFlow_Scheduler$add_start,
    remove_start: jsWorkFlow_Scheduler$remove_start,
    add_pause: jsWorkFlow_Scheduler$add_pause,
    remove_pause: jsWorkFlow_Scheduler$remove_pause,
    add_resume: jsWorkFlow_Scheduler$add_resume,
    remove_resume: jsWorkFlow_Scheduler$remove_resume,
    add_stop: jsWorkFlow_Scheduler$add_stop,
    remove_stop: jsWorkFlow_Scheduler$remove_stop
};


jso.registerClass(jso.setTypeName(jsWorkFlow.Scheduler, 'jsWorkFlow.Scheduler'));




//find root ns
var jsWorkFlow = jso.ns('jsWorkFlow');

//  provide the activity that use by other user.
//  main API:
//      registerTemplate(jsDoc)  --> register activity template
//      getTemplate(id)            --> get activity template
//      clearTemplates()           --> clear templates
//      registerObject(id, obj)    --> register object single instance
//      clearObjects()             --> clear activity registry
//      *tryGetObject(id)          --> from registry, or create from template and add to activity registry 
//      *getObject(id)             --> from registry, or create from template and add to activity registry 
//      *tryCreateObject(id)       --> from template, and don't add to activity registry
//      *createObject(id)          --> from template, and don't add to activity registry
//
//[
//{
//    //document of descript an activity
//    'id': 'xxxx',               //>>>> is the uniqueName in activity factory
//    'type': 'jsWorkFlow.Activities.SequenceActivity',
//    'properties':{
//    	'name': 'xxxx',		//>>>> the display name of activity
//        'activities': [value1, value1, value1], //should use the name of activity
//        'other_prop': 'xxx'
//    }
//	'constructors':[value1, value2, value3],
//},
//{
//    //document of descript an instance
//    'id': 'xxxx',               //>>>> is the uniqueName in activity factory
//    'type': 'jsWorkFlow.Instance',
//    'properties':{
//    	'name': 'xxxx',		//>>>> the display name of activity
//      'activities': ['activity1', 'activity1', 'activity1'], //should use the name of activity
//      'other_prop': 'xxx'
//    }
//},
//{
//    //document of descript an application
//    'id': 'xxxx',               //>>>> is the uniqueName in activity factory
//    'type': 'jsWorkFlow.Application',
//    'properties':{
//    	'name': 'xxxx',		//>>>> the display name of activity
//        'activities': ['activity1', 'activity1', 'activity1'], //should use the name of activity
//        'other_prop': 'xxx'
//    }
//}
//]
//
//value description:
//  "string", 123, Date, Number, Array, Object are all simple value.
//  if object has __v propery with false boolean value, is a simple value.
//  if object has __v propery with true boolean value, see follow:
//	    1. with ref property with string value, is activity instance reference.
//	    2. with val property with json value, the val is simple value

jsWorkFlow.ActivityFactory = function jsWorkFlow_ActivityFactory() {
    this._templates = {};
    this._registry = {};
};

function jsWorkFlow_ActivityFactory$dispose() {
    this._templates = {};
    this._registry = {};
}

function _jsw_af$verifyTemplate(jsDoc) {
    if (!jsDoc) {
        throw jso.errorInvalidOperation("invalid template");
    }

    if (!jsDoc["id"] || !jsDoc["type"]) {
        throw jso.errorInvalidOperation("invalid template");
    }

    //add other rule here
}

function jsWorkFlow_ActivityFactory$registerTemplate(jsDoc) {
    if(!jsDoc) {
        throw jso.errorArgument("jsDoc", "undefined");
    }

    if(!jso.isArray(jsDoc)) {
        jsDoc = [jsDoc];
    }

    for(var i=0, ilen=jsDoc.length; i<ilen;i++) {
        var curDoc = jsDoc[i];

        _jsw_af$verifyTemplate(curDoc);
        this._templates[curDoc.id] = curDoc;

    }
    
    //OK
}

function jsWorkFlow_ActivityFactory$getTemplate(id) {
    return this._templates[id];
}

function jsWorkFlow_ActivityFactory$clearTemplates() {
    this._templates = {};
}

function jsWorkFlow_ActivityFactory$registerObject(id, obj) {
    this._registry[id] = obj;
}

function jsWorkFlow_ActivityFactory$clearObjects() {
    this._registry = {};
}

//from registry, or create from template and add to activity registry
function jsWorkFlow_ActivityFactory$tryGetObject(id) {
    var obj = this._registry[id];

    if (!obj) {
        obj = this.tryCreateObject(id);
        if (obj) {
            this._registry[id] = obj;
        }
    }

    return obj;

}

//from registry, or create from template and add to activity registry
function jsWorkFlow_ActivityFactory$getObject(id) {
    var obj = this._registry[id];

    if (!obj) {
        obj = this.createObject(id);
        if (obj) {
            this._registry[id] = obj;
        }
    }

    return obj;

}

//process ref and val property
function _jsw_af$createValueByDesc(valueDesc, objFactory) {
    var retval = null;

    if (valueDesc.ref) {
        //get object instance from factory
        retval = objFactory.getObject(valueDesc.ref);
    }
    else if (valueDesc.val) {
        retval = valueDesc.val;
    }

    return retval;
}

//process array, object and pure value
//value description:
//  "string", 123, Date, Number, Array, Object are all simple value.
//      1. when is not strict mode, string value may is a ID of object.
//  if object has __v propery with false boolean value, is a simple value.
//  if object has __v propery with true boolean value, see follow:
//	    1. with ref property with string value, is activity instance reference.
//	    2. with val property with json value, the val is simple value
function _jsw_af$createValue(valueDesc, objFactory, strict) {
    var retval = null;

    if(!valueDesc) {
        return retval;
    }

    if (jso.isArray(valueDesc)) {
        //build array value 
        retval = [];
        for (var i = 0, ilen = valueDesc.length; i < ilen; i++) {
            //consider every element is a value descriptor
            var curValDesc = valueDesc[i];
            var curVal = _jsw_af$createValue(curValDesc, objFactory, strict);
            jso.arrayAdd(retval, curVal);
        }
    } else if (jso.isPureObject(valueDesc)) {
        if (!!valueDesc['__v']) {
            //is a value descriptor
            retval = _jsw_af$createValueByDesc(valueDesc, objFactory);
        }
        else {
            //is a simple value object 
            //build object value 
            retval = {};
            for (var key in valueDesc) {
                if (key === '__v') {
                    //ignore the value descriptor flag.
                    continue;
                }
                //consider every key is a value descriptor
                var curValDesc = valueDesc[key];
                var curVal = _jsw_af$createValue(curValDesc, objFactory, strict);
                retval[key] = curVal;
            }
        }
    } else {
        //string value may represent a object reference when is not strict mode
        if (jso.isString(valueDesc) && !strict) {
            //try get object from factory
            var obj = objFactory.tryGetObject(valueDesc);

            if (obj) {
                retval = obj;
            }
            else {
                retval = valueDesc;
            }
        }
        else {
            //simple value, just return it
            retval = valueDesc;
        }
    }

    return retval;

}

function _jsw_af$applyProperties(ins, props) {
    for (var prop in props) {
        var setProp = "set_"+prop;
        var method = ins[setProp];
        var propValue = props[prop];

        if (!method) {
            throw jso.errorNotExists("property:[" + setProp + "]");
        }

        //set property value
        method.apply(ins, [propValue]);
    }
}

//create a object instance of a type.
function _jsw_af$createObject(jsDoc, objFactory) {

    //first of all, get type from jsDoc
    var typeName = jsDoc['type'];
    var strict = !!jsDoc['strict'];

    //get constructor params of doc
    var constructorsDesc = jsDoc['constructors'];

    //properties of doc
    var propertiesDesc = jsDoc['properties'];

    var type = jso.getType(typeName);

    if (!type) {
        throw jso.errorNotExists("type:[" + typeName + "]");
    }

    //build constructor
    var constrctors = _jsw_af$createValue(constructorsDesc, objFactory, strict);

    //build properties
    var properties = {};
    for (var key in propertiesDesc) {
        var curPropValueDesc = propertiesDesc[key];
        var curPropValue = _jsw_af$createValue(curPropValueDesc, objFactory, strict);
        properties[key] = curPropValue;
    }

    //create object and set properties
    var instance = jso.createInstance(type, constrctors);

    //apply properties
    _jsw_af$applyProperties(instance, properties);

    return instance;
}

function jsWorkFlow_ActivityFactory$tryCreateObject(id) {
    var jsDoc = this.getTemplate(id);

    if (!jsDoc) {
        return null;
    }

    //create activity by template
    var obj = _jsw_af$createObject(jsDoc, this);

    return obj;
}


//from template, and don't add to activity registry
function jsWorkFlow_ActivityFactory$createObject(id) {
    var jsDoc = this.getTemplate(id);

    if (!jsDoc) {
        throw jso.errorNotExists("id:[" + id + "]");
    }

    //create activity by template
    var obj = _jsw_af$createObject(jsDoc, this);

    return obj;
}

jsWorkFlow.ActivityFactory.prototype = {
    _templates: null,
    _registry: null,
    //method
    dispose: jsWorkFlow_ActivityFactory$dispose,
    registerTemplate: jsWorkFlow_ActivityFactory$registerTemplate,
    getTemplate: jsWorkFlow_ActivityFactory$getTemplate,
    clearTemplates: jsWorkFlow_ActivityFactory$clearTemplates,
    registerObject: jsWorkFlow_ActivityFactory$registerObject,
    clearObjects: jsWorkFlow_ActivityFactory$clearObjects,
    tryGetObject: jsWorkFlow_ActivityFactory$tryGetObject,
    getObject: jsWorkFlow_ActivityFactory$getObject,
    tryCreateObject: jsWorkFlow_ActivityFactory$tryCreateObject,
    createObject: jsWorkFlow_ActivityFactory$createObject
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ActivityFactory, 'jsWorkFlow.ActivityFactory'));



//require namsepace
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//NullLogger，提供jsWorkFlow的默认的LOG
//
jsWorkFlow.NullLogger = function jsWorkFlow_NullLogger() {
};

//提供一个空实现，什么都不做
function jsWorkFlow_NullLogger$dummy() {
}

jsWorkFlow.NullLogger.prototype = {
    trace: jsWorkFlow_NullLogger$dummy,
    debug: jsWorkFlow_NullLogger$dummy,
    info: jsWorkFlow_NullLogger$dummy,
    warn: jsWorkFlow_NullLogger$dummy,
    error: jsWorkFlow_NullLogger$dummy,
    fatal: jsWorkFlow_NullLogger$dummy
};

jso.registerClass(jso.setTypeName(jsWorkFlow.NullLogger, 'jsWorkFlow.NullLogger'));

//////////////////////////////////////////////////////////////////////////////////////////
//ConsoleLogger，提供jsWorkFlow的默认的LOG
//
jsWorkFlow.ConsoleLogger = function jsWorkFlow_ConsoleLogger() {
};

function jsWorkFlow_ConsoleLogger$_log(level, message) {
    console.log('[' + level + ']+%s', message);
}

function jsWorkFlow_ConsoleLogger$trace(message) {
    this._log('trace', message);
}
function jsWorkFlow_ConsoleLogger$debug(message) {
    this._log('debug', message);
}
function jsWorkFlow_ConsoleLogger$info(message) {
    this._log('info', message);
}
function jsWorkFlow_ConsoleLogger$warn(message) {
    this._log('warn', message);
}
function jsWorkFlow_ConsoleLogger$error(message) {
    this._log('error', message);
}
function jsWorkFlow_ConsoleLogger$fatal(message) {
    this._log('fatal', message);
}

jsWorkFlow.ConsoleLogger.prototype = {
    _log: jsWorkFlow_ConsoleLogger$_log,
    trace: jsWorkFlow_ConsoleLogger$trace,
    debug: jsWorkFlow_ConsoleLogger$debug,
    info: jsWorkFlow_ConsoleLogger$info,
    warn: jsWorkFlow_ConsoleLogger$warn,
    error: jsWorkFlow_ConsoleLogger$error,
    fatal: jsWorkFlow_ConsoleLogger$fatal
};

jso.registerClass(jso.setTypeName(jsWorkFlow.ConsoleLogger, 'jsWorkFlow.ConsoleLogger'));


//////////////////////////////////////////////////////////////////////////////////////////
//jsWorkFlow的LOG的适配
//
//TO 开发者：
//    LOG类用于适配jsWorkFlow对外部LOG体系的依赖，并对内提供LOG的基础API
//
var _jwf$logger = null;

function _jwf$config_log() {
    var log = null;

    //log4javascript的适配
    if (typeof (log4javascript) !== "undefined") {
        var log = log4javascript.getLogger();

        var popUpAppender = new log4javascript.PopUpAppender();
        var popUpLayout = new log4javascript.PatternLayout("%d{HH:mm:ss} %-5p - %m%n");
        popUpAppender.setLayout(popUpLayout);
        log.addAppender(popUpAppender);

    }
//    else if ((typeof (console) != 'undefined') && (typeof (console.log) != 'undefined')) {
//        log = new jsWorkFlow.ConsoleLogger();
//    }
    else {
        //如果没有log4javascript，使用NullLogger来替换
        log = new jsWorkFlow.NullLogger();
    }

    _jwf$logger = log;
}

//取全局的LOG
function jwf$getLogger() {
    return _jwf$logger;
}

//config log here
_jwf$config_log();




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//DelayActivity，延迟执行时间
//
//TO 开发者：
//    DelayActivity延迟（推迟）执行的时间，单位是ms。
jsWorkFlow.Activities.DelayActivity = function jsWorkFlow_Activities_DelayActivity(delayTime) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DelayActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.DelayActivity, this);
    this._delayTime = delayTime;
};

function jsWorkFlow_Activities_DelayActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DelayActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.DelayActivity, this, 'dispose');
}

function jsWorkFlow_Activities_DelayActivity$get_delayTime() {
    return this._delayTime;
}

function jsWorkFlow_Activities_DelayActivity$set_delayTime(value) {
    this._delayTime = value;
}

//activity的恢复
function jsWorkFlow_Activities_DelayActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DelayActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.DelayActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.DelayActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    this.set_delayTime(serializeContext['delayTime']);

}

//activity的序列化
function jsWorkFlow_Activities_DelayActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DelayActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.DelayActivity';

    //保存自身
    serializeContext['delayTime'] = this.get_delayTime();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.DelayActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_DelayActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DelayActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.DelayActivity, this, 'execute', [context]);

    //通过lamda设置回调
    context._timeoutID = jsWorkFlow.setTimeout(function () {
        //LOG 
        $jwf.endActivity(context);
        jsWorkFlow.clearTimeout(context._timeoutID);
    }, this._delayTime);
}

jsWorkFlow.Activities.DelayActivity.prototype = {
    _delayTime: null,
    dispose: jsWorkFlow_Activities_DelayActivity$dispose,
    //property
    get_delayTime: jsWorkFlow_Activities_DelayActivity$get_delayTime,
    set_delayTime: jsWorkFlow_Activities_DelayActivity$set_delayTime,
    //method
    loadSerializeContext: jsWorkFlow_Activities_DelayActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_DelayActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_DelayActivity$execute

};

jso.registerClass(jso.setTypeName(jsWorkFlow.Activities.DelayActivity, 'jsWorkFlow.Activities.DelayActivity'), jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//EvalExprActivity
//
//TO 开发者：
//    执行一个字符串表示的javascript表达式，并返回其执行结果。
//    对于动态语言，天然存在的表达式的执行eval是其自然的优势，将其引入到activity之中会丰富
//workflow的表达能力。
//
jsWorkFlow.Activities.EvalExprActivity = function jsWorkFlow_Activities_EvalExprActivity(expr) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EvalExprActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.EvalExprActivity, this);

    this.set_expr(expr);
};

function jsWorkFlow_Activities_EvalExprActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EvalExprActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.EvalExprActivity, this, 'dispose');
}

function jsWorkFlow_Activities_EvalExprActivity$get_expr() {
    return this._expr;
}

function jsWorkFlow_Activities_EvalExprActivity$set_expr(value) {
    if (typeof (value) === "string") {
        this._expr = value;
    } else {
        if ((typeof (value) === "undefined") || (value === null)) {
            this._expr = "";
        }
        else {
            throw jso.errorInvalidOperation();
        }
    }
}

//activity的恢复
function jsWorkFlow_Activities_EvalExprActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EvalExprActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.EvalExprActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.EvalExprActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    this.set_expr(serializeContext['expr']);
}

//activity的序列化
function jsWorkFlow_Activities_EvalExprActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EvalExprActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.EvalExprActivity';

    //保存自身
    serializeContext['expr'] = this.get_expr();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.EvalExprActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_EvalExprActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EvalExprActivity execute!");


    jso.callBaseMethod(jsWorkFlow.Activities.EvalExprActivity, this, 'execute', [context]);

    //如果有表达式，执行，并将结果保存到context之中
    if (this._expr.length > 0) {
        var result = eval(this._expr);

        context.set_result(result);
    }

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.EvalExprActivity.prototype = {
    _expr: null,
    dispose: jsWorkFlow_Activities_EvalExprActivity$dispose,
    //property
    get_expr: jsWorkFlow_Activities_EvalExprActivity$get_expr,
    set_expr: jsWorkFlow_Activities_EvalExprActivity$set_expr,
    //method
    loadSerializeContext: jsWorkFlow_Activities_EvalExprActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_EvalExprActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_EvalExprActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.EvalExprActivity, 'jsWorkFlow.Activities.EvalExprActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//FunctionActivityEventArgs，提供jsWorkFlow的FunctionActivity的事件响应参数
//
// TO 开发者：
//
jsWorkFlow.Activities.FunctionActivityEventArgs = function jsWorkFlow_Activities_FunctionActivityEventArgs(context, callbackData) {
    jso.initializeBase(jsWorkFlow.Activities.FunctionActivityEventArgs, this, [context, callbackData]);
};

function jsWorkFlow_Activities_FunctionActivityEventArgs$dispose() {
    jso.callBaseMethod(jsWorkFlow.Activities.FunctionActivityEventArgs, this, 'dispose');
}

function jsWorkFlow_Activities_FunctionActivityEventArgs$get_callbackData() {
    return this.get_data();
}

jsWorkFlow.Activities.FunctionActivityEventArgs.prototype = {
    dispose: jsWorkFlow_Activities_FunctionActivityEventArgs$dispose,
    //property
    get_callbackData: jsWorkFlow_Activities_FunctionActivityEventArgs$get_callbackData
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.FunctionActivityEventArgs, 'jsWorkFlow.Activities.FunctionActivityEventArgs'), 
    jsWorkFlow.ActivityEventArgs);


//////////////////////////////////////////////////////////////////////////////////////////
//FunctionActivity，将function call封装到一个activity之中
//
//TO 开发者：
//    function的调用形式为function fun(FunctionActivityEventArgs) { ... }。
//    load & save SerializeContext目前只支持具名函数的序列化&恢复
//
jsWorkFlow.Activities.FunctionActivity = function jsWorkFlow_Activities_FunctionActivity(func, callbackData) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.FunctionActivity create!");

    this._func = func;
    this._callbackData = callbackData;
    jso.initializeBase(jsWorkFlow.Activities.FunctionActivity, this);
};

function jsWorkFlow_Activities_FunctionActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.FunctionActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.FunctionActivity, this, 'dispose');
}

function jsWorkFlow_Activities_FunctionActivity$get_func() {
    return this._func;
}

function jsWorkFlow_Activities_FunctionActivity$set_func(value) {
    this._func = value;
}

//activity的恢复****目前只支持具名函数的序列化&恢复****
function jsWorkFlow_Activities_FunctionActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.FunctionActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.FunctionActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.FunctionActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    this.set_func(eval(serializeContext['func']));
}

//activity的序列化****目前只支持具名函数的序列化&恢复****
function jsWorkFlow_Activities_FunctionActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.FunctionActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.FunctionActivity';

    //保存自身
    var func = this.get_func();
    var funcName = "";

    if (func) {
        var funcBody = func.toString();
        funcName = funcBody.substring(funcBody.indexOf("function")+8, funcBody.indexOf("("));
    }
    serializeContext['func'] = funcName;

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.FunctionActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_FunctionActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.FunctionActivity execute!");

    //执行基类execute方法，会进入start状态
    jso.callBaseMethod(jsWorkFlow.Activities.FunctionActivity, this, 'execute', [context]);

    //同步执行function
    var eventArgs = new jsWorkFlow.Activities.FunctionActivityEventArgs(context, this._callbackData);

    var result = this._func(eventArgs);

    //将执行结果放到context之中
    context.set_result(result);

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.FunctionActivity.prototype = {
    _func: null,
    _callbackData: null,
    dispose: jsWorkFlow_Activities_FunctionActivity$dispose,
    //property
    get_func: jsWorkFlow_Activities_FunctionActivity$get_func,
    set_func: jsWorkFlow_Activities_FunctionActivity$set_func,
    //method
    loadSerializeContext: jsWorkFlow_Activities_FunctionActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_FunctionActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_FunctionActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.FunctionActivity, 'jsWorkFlow.Activities.FunctionActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//IfElseActivity
//
//TO 开发者：
//    IfElseActivity是一个if...then...else分支的activity，包含3个部分，conditionActivity、
//thenActivity和elseActivity，conditionActivity提供if所需要的分支条件，如果分支条件返回结果
//评估是true，执行thenActivity，否则执行elseActivity。
//    conditionActivity、thenActivity和elseActivity都可以为空。
//    IfElseActivity将执行的activity(thenActivity或elseActivity)作为自己的返回值。
jsWorkFlow.Activities.IfElseActivity = function jsWorkFlow_Activities_IfElseActivity(conditionActivity, thenActivity, elseActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.IfElseActivity, this);

    this.set_conditionActivity(conditionActivity);
    this.set_thenActivity(thenActivity);
    this.set_elseActivity(elseActivity);

    this._doEvalConditionCompleteHandler = jso.createDelegate(this, this.doEvalConditionCompleteHandler);
    this._doExecuteBodyCompleteHandler = jso.createDelegate(this, this.doExecuteBodyCompleteHandler);
};

function jsWorkFlow_Activities_IfElseActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity dispose!");

    this._conditionActivity = null;
    this._elseActivity = null;
    this._thenActivity = null;
    jso.callBaseMethod(jsWorkFlow.Activities.IfElseActivity, this, 'dispose');
}

function jsWorkFlow_Activities_IfElseActivity$get_conditionActivity() {
    return this._conditionActivity;
}

function jsWorkFlow_Activities_IfElseActivity$set_conditionActivity(value) {
    this._conditionActivity = value;
}

function jsWorkFlow_Activities_IfElseActivity$get_thenActivity() {
    return this._thenActivity;
}

function jsWorkFlow_Activities_IfElseActivity$set_thenActivity(value) {
    this._thenActivity = value;
}

function jsWorkFlow_Activities_IfElseActivity$get_elseActivity() {
    return this._elseActivity;
}

function jsWorkFlow_Activities_IfElseActivity$set_elseActivity(value) {
    this._elseActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_IfElseActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.IfElseActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.IfElseActivity, this, 'loadSerializeContext', [baseSerializeContext]);


    //恢复自身
    var conditionActivitySC = serializeContext['conditionActivity'];
    var conditionActivity = $jwf.loadActivity(conditionActivitySC);
    this.set_conditionActivity(conditionActivity);

    var thenActivitySC = serializeContext['thenActivity'];
    var thenActivity = $jwf.loadActivity(thenActivitySC);
    this.set_thenActivity(thenActivity);

    var elseActivitySC = serializeContext['elseActivity'];
    var elseActivity = $jwf.loadActivity(elseActivitySC);
    this.set_elseActivity(elseActivity);

}

//activity的序列化
function jsWorkFlow_Activities_IfElseActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.IfElseActivity';

    //保存自身
    serializeContext['conditionActivity'] = $jwf.saveActivity(this.get_conditionActivity());
    serializeContext['thenActivity'] = $jwf.saveActivity(this.get_thenActivity());
    serializeContext['elseActivity'] = $jwf.saveActivity(this.get_elseActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.IfElseActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}


function jsWorkFlow_Activities_IfElseActivity$doEvalCondition(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity doEvalCondition!");

    //如果没有设置条件，认为为false，执行else分支
    var activity = this._conditionActivity;

    if (!activity) {
        this.doExecuteBody(context, false);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalConditionCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_IfElseActivity$doEvalConditionCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity doEvalConditionCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var condition = context.get_result();

    //将condition传递给doExecuteBody继续执行
    this.doExecuteBody(parentContext, condition);
}

function jsWorkFlow_Activities_IfElseActivity$doExecuteBody(context, condition) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity doExecuteBody!");

    //如果没有设置条件，认为为false，执行else分支
    var activity = (condition)? this._thenActivity: this._elseActivity;

    log.debug("condition is:[" + !!condition + "!");


    if (!activity) {
        //没有activity，结束执行
        $jwf.endActivity(context);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecuteBodyCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_IfElseActivity$doExecuteBodyCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity doExecuteBodyCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //将activity的最后的执行结果作为当前activity的执行结果
    var result = context.get_result();
    parentContext.set_result(result);

    //结束activity的执行
    $jwf.endActivity(parentContext);
    return;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_IfElseActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.IfElseActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.IfElseActivity, this, 'execute', [context]);

    //从执行条件开始kick out
    this.doEvalCondition(context);
}

jsWorkFlow.Activities.IfElseActivity.prototype = {
    _conditionActivity: null,
    _thenActivity: null,
    _elseActivity: null,
    _doEvalConditionCompleteHandler: null,
    _doExecuteBodyCompleteHandler: null,
    dispose: jsWorkFlow_Activities_IfElseActivity$dispose,
    //property
    get_conditionActivity: jsWorkFlow_Activities_IfElseActivity$get_conditionActivity,
    set_conditionActivity: jsWorkFlow_Activities_IfElseActivity$set_conditionActivity,
    get_thenActivity: jsWorkFlow_Activities_IfElseActivity$get_thenActivity,
    set_thenActivity: jsWorkFlow_Activities_IfElseActivity$set_thenActivity,
    get_elseActivity: jsWorkFlow_Activities_IfElseActivity$get_elseActivity,
    set_elseActivity: jsWorkFlow_Activities_IfElseActivity$set_elseActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_IfElseActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_IfElseActivity$saveSerializeContext,
    doEvalCondition: jsWorkFlow_Activities_IfElseActivity$doEvalCondition,
    doEvalConditionCompleteHandler: jsWorkFlow_Activities_IfElseActivity$doEvalConditionCompleteHandler,
    doExecuteBody: jsWorkFlow_Activities_IfElseActivity$doExecuteBody,
    doExecuteBodyCompleteHandler: jsWorkFlow_Activities_IfElseActivity$doExecuteBodyCompleteHandler,
    execute: jsWorkFlow_Activities_IfElseActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.IfElseActivity, 'jsWorkFlow.Activities.IfElseActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//NoopActivity
//
//TO 开发者：
//    一个空操作(noop)的activity，仅用于展示activity得到基本控制流程，可以当成activity的占位
//符使用。
//
jsWorkFlow.Activities.NoopActivity = function jsWorkFlow_Activities_NoopActivity() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.NoopActivity create!");


    jso.initializeBase(jsWorkFlow.Activities.NoopActivity, this);

};

function jsWorkFlow_Activities_NoopActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.NoopActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.NoopActivity, this, 'dispose');
}


//activity的恢复
function jsWorkFlow_Activities_NoopActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.NoopActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.NoopActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.NoopActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身

}

//activity的序列化
function jsWorkFlow_Activities_NoopActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.NoopActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.NoopActivity';

    //保存自身

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.NoopActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_NoopActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.NoopActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.NoopActivity, this, 'execute', [context]);

    //TODO:
    //    LOG noop message!

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.NoopActivity.prototype = {
    dispose: jsWorkFlow_Activities_NoopActivity$dispose,
    //property
    //method
    loadSerializeContext: jsWorkFlow_Activities_NoopActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_NoopActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_NoopActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.NoopActivity, 'jsWorkFlow.Activities.NoopActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//GetExceptionActivity
//
//TO 开发者：
//    GetExceptionActivity用于在tryCatch的Catch中获取异常的activity，可以根据异常信息进行分类处理，
//或重新的发布异常。
//    GetExceptionActivity从当前context向上搜索，查找状态是error的context，并返回其errorInfo中的，
//exception属性。
//

jsWorkFlow.Activities.GetExceptionActivity = function jsWorkFlow_Activities_GetExceptionActivity() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetExceptionActivity create!");


    jso.initializeBase(jsWorkFlow.Activities.GetExceptionActivity, this);

};

function jsWorkFlow_Activities_GetExceptionActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetExceptionActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetExceptionActivity, this, 'dispose');
}


//activity的恢复
function jsWorkFlow_Activities_GetExceptionActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetExceptionActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.GetExceptionActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.GetExceptionActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身

}

//activity的序列化
function jsWorkFlow_Activities_GetExceptionActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetExceptionActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.GetExceptionActivity';

    //保存自身

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.GetExceptionActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_GetExceptionActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetExceptionActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetExceptionActivity, this, 'execute', [context]);

    //获取error信息
    var exception = null;
    var errorInfo = $jwf._getErrorInfo(context);
    if (errorInfo) {
        exception = errorInfo.get_exception();
    }

    //将exception信息放置到context
    context.set_result(exception);

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.GetExceptionActivity.prototype = {
    dispose: jsWorkFlow_Activities_GetExceptionActivity$dispose,
    //property
    //method
    loadSerializeContext: jsWorkFlow_Activities_GetExceptionActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_GetExceptionActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_GetExceptionActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.GetExceptionActivity, 'jsWorkFlow.Activities.GetExceptionActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//DataContextLayer枚举，表示可以定义的数据层
jsWorkFlow.DataContextLayer = function jsWorkFlow_DataContextLayer() {
    throw new Error.notImplemented();
};

//系统默认的activity的状态
jsWorkFlow.DataContextLayer.prototype = {
    auto: 0,          //自动，定义等同于activity，获取会搜索
    local: 0,         //只在当前的activity层级上定义和搜索
    application: 1,   //APP的范围内共享
    global: 2         //全局，跨APP
};

jso.registerEnum(jso.setTypeName(jsWorkFlow.DataContextLayer, 'jsWorkFlow.DataContextLayer'));


//////////////////////////////////////////////////////////////////////////////////////////
//DefineContextDataActivity
//
//TO 开发者：
//    是定义ContextData的activity，用于定义数据，类似变量，存储activity执行期间的数据。
//    DefineContextDataActivity的定义选择中，dataKey属性只支持常量，否则会将复杂性带入到工作流
//之中，不利于上下游在数据层面协同工作。dataValueActivity属性是activity类型，支持数据根据上下文
//做初始化。dataContextLayer和isPrivate是数据定义层面的选项。
//    注意，DefineContextDataActivity的执行结果是要将数据存放在parent的context之中。
//
jsWorkFlow.Activities.DefineContextDataActivity = function jsWorkFlow_Activities_DefineContextDataActivity(dataKey, dataValueActivity, isPrivate, dataContextLayer) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity execute!");

    jso.initializeBase(jsWorkFlow.Activities.DefineContextDataActivity, this);

    if (typeof (dataContextLayer) === "undefined") {
        //默认数据存储在activity层级上
        dataContextLayer = jsWorkFlow.DataContextLayer.auto;
    }

    this._dataKey = dataKey;
    this._dataValueActivity = dataValueActivity;
    this._dataContextLayer = dataContextLayer;
    this._isPrivate = !!isPrivate;

    this._doEvalDataValueActivityCompleteHandler = jso.createDelegate(this, this.doEvalDataValueActivityCompleteHandler);

};

function jsWorkFlow_Activities_DefineContextDataActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.DefineContextDataActivity, this, 'dispose');

    this._dataKey = null;
    this._dataValueActivity = null;
}

function jsWorkFlow_Activities_DefineContextDataActivity$get_dataKey() {
    return this._dataKey;
}

function jsWorkFlow_Activities_DefineContextDataActivity$set_dataKey(value) {
    this._dataKey = value;
}

function jsWorkFlow_Activities_DefineContextDataActivity$get_dataValueActivity() {
    return this._dataValueActivity;
}

function jsWorkFlow_Activities_DefineContextDataActivity$set_dataValueActivity(value) {
    this._dataValueActivity = value;
}

function jsWorkFlow_Activities_DefineContextDataActivity$get_dataContextLayer() {
    return this._dataContextLayer;
}

function jsWorkFlow_Activities_DefineContextDataActivity$set_dataContextLayer(value) {
    this._dataContextLayer = value;
}

function jsWorkFlow_Activities_DefineContextDataActivity$get_isPrivate() {
    return this._isPrivate;
}

function jsWorkFlow_Activities_DefineContextDataActivity$set_isPrivate(value) {
    this._isPrivate = !!value;
}

//activity的恢复
function jsWorkFlow_Activities_DefineContextDataActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.DefineContextDataActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.DefineContextDataActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    //dataKey
    var dataKey = serializeContext["dataKey"];
    this.set_dataKey(dataKey);
    //dataValueActivity
    var dataValueActivitySC = serializeContext["dataValueActivity"];
    var dataValueActivity = $jwf.loadActivity(dataValueActivitySC);
    this.set_dataValueActivity(dataValueActivity);
    //dataContextLayer
    var dataContextLayer = serializeContext["dataContextLayer"];
    this.set_dataContextLayer(dataContextLayer);
    //isPrivate
    var isPrivate = serializeContext["isPrivate"];
    this.set_isPrivate(isPrivate);
}

//activity的序列化
function jsWorkFlow_Activities_DefineContextDataActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.DefineContextDataActivity';

    //保存自身
    //dataKey
    serializeContext["dataKey"] = this.get_dataKey();
    //dataValueActivity
    var dataValueActivity = this.get_dataValueActivity();
    serializeContext["dataValueActivity"] = $jwf.saveActivity(dataValueActivity);
    //dataContextLayer
    serializeContext["dataContextLayer"] = this.get_dataContextLayer();
    //isPrivate
    serializeContext["isPrivate"] = this.get_isPrivate();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.DefineContextDataActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_DefineContextDataActivity$doInitContextDataValue(application, value) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity doInitContextDataValue!");

    //使用给出的值来初始化contextData
    var dataContextLayer = this._dataContextLayer;
    var dataKey  = this._dataKey;
    var isPrivate  = this._isPrivate;
    var dataContext = null;

    //根据dataContextLayer来初始化数据项
    if ((jsWorkFlow.DataContextLayer.auto === dataContextLayer) ||
    (jsWorkFlow.DataContextLayer.local === dataContextLayer)) {
        //activity层级的数据定义，要跳过当前（DefineContextDataActivity）层，将数据存放上一层之中
        dataContext = application.internalContextAt(1);
    }
    else if (jsWorkFlow.DataContextLayer.application === dataContextLayer) {
        //app层级
        dataContext = application.get_appContext();
    }
    else if (jsWorkFlow.DataContextLayer.global === dataContextLayer) {
        //global层级
        dataContext = application.get_globalContext();
    }
    else {
        //LOG Here!
        throw jso.errorInvalidOperation("Unknown dataContextLayer value [" + dataContextLayer + "]!");
    }
    //根据
    var isPublic = !isPrivate;
    var switchVisiblity = (dataContext.get_defaultVisibilityIsPublic() === isPublic) ? false : true;
    dataContext.setData(dataKey, value, switchVisiblity);
}

function jsWorkFlow_Activities_DefineContextDataActivity$doEvalDataValueActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity doEvalDataValueActivityCompleteHandler!");
    //post complete事件
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var application = parentContext.get_application();

    //默认值是null
    var result = null;

    //cleanup old executor
    executor.remove_postComplete(this._doEvalDataValueActivityCompleteHandler);

    //set last context's result as current context's result
    result = context.get_result();

    //将parent的result设置为求值的结果
    parentContext.set_result(result);

    //设置ContextData的值
    this.doInitContextDataValue(application, result);

    //执行完毕，结束activity
    $jwf.endActivity(parentContext);

}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_DefineContextDataActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.DefineContextDataActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.DefineContextDataActivity, this, 'execute', [context]);

    //对dataValueActivity求值
    //将值放到dataKey之中

    //选择activity，并执行
    var activity = this._dataValueActivity;
    var application = context.get_application();

    if (!activity) {
        //空的activity，认为是null值
        this.doInitContextDataValue(application, null);
        //执行完毕，结束activity
        $jwf.endActivity(context);
        return;
    }

    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalDataValueActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

jsWorkFlow.Activities.DefineContextDataActivity.prototype = {
    _dataKey: null,
    _dataValueActivity: null,
    _dataContextLayer: 0,
    _isPrivate: false,
    dispose: jsWorkFlow_Activities_DefineContextDataActivity$dispose,
    //property
    get_dataKey: jsWorkFlow_Activities_DefineContextDataActivity$get_dataKey,
    set_dataKey: jsWorkFlow_Activities_DefineContextDataActivity$set_dataKey,
    get_dataValueActivity: jsWorkFlow_Activities_DefineContextDataActivity$get_dataValueActivity,
    set_dataValueActivity: jsWorkFlow_Activities_DefineContextDataActivity$set_dataValueActivity,
    get_dataContextLayer: jsWorkFlow_Activities_DefineContextDataActivity$get_dataContextLayer,
    set_dataContextLayer: jsWorkFlow_Activities_DefineContextDataActivity$set_dataContextLayer,
    get_isPrivate: jsWorkFlow_Activities_DefineContextDataActivity$get_isPrivate,
    set_isPrivate: jsWorkFlow_Activities_DefineContextDataActivity$set_isPrivate,
    //method
    loadSerializeContext: jsWorkFlow_Activities_DefineContextDataActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_DefineContextDataActivity$saveSerializeContext,
    doInitContextDataValue: jsWorkFlow_Activities_DefineContextDataActivity$doInitContextDataValue,
    doEvalDataValueActivityCompleteHandler: jsWorkFlow_Activities_DefineContextDataActivity$doEvalDataValueActivityCompleteHandler,
    execute: jsWorkFlow_Activities_DefineContextDataActivity$execute

};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.DefineContextDataActivity, 'jsWorkFlow.Activities.DefineContextDataActivity'), 
    jsWorkFlow.Activity);

//////////////////////////////////////////////////////////////////////////////////////////
//GetContextDataActivity
//
//TO 开发者：
//    获取Context中数据相值的activity。对于指定auto类型的数据获取，只能访问public的数据；对于其他的类型，
//只访问指定的数据容器，可以看到私有的数据项。
//
jsWorkFlow.Activities.GetContextDataActivity = function jsWorkFlow_Activities_GetContextDataActivity(dataKey, dataContextLayer) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetContextDataActivity execute!");

    jso.initializeBase(jsWorkFlow.Activities.GetContextDataActivity, this);

    if (typeof (dataContextLayer) === "undefined") {
        dataContextLayer = jsWorkFlow.DataContextLayer.auto;
    }

    this._dataKey = dataKey;
    this._dataContextLayer = dataContextLayer;
};

function jsWorkFlow_Activities_GetContextDataActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetContextDataActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetContextDataActivity, this, 'dispose');
}

function jsWorkFlow_Activities_GetContextDataActivity$get_dataKey() {
    return this._dataKey;
}
function jsWorkFlow_Activities_GetContextDataActivity$set_dataKey(value) {
    this._dataKey = value;
}

function jsWorkFlow_Activities_GetContextDataActivity$get_dataContextLayer() {
    return this._dataContextLayer;
}
function jsWorkFlow_Activities_GetContextDataActivity$set_dataContextLayer(value) {
    this._dataContextLayer = value;
}


//activity的恢复
function jsWorkFlow_Activities_GetContextDataActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetContextDataActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.GetContextDataActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.GetContextDataActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var dataKey = serializeContext["dataKey"];
    this.set_dataKey(dataKey);

    var dataContextLayer = serializeContext["dataContextLayer"];
    this.set_dataContextLayer(dataContextLayer);
}

//activity的序列化
function jsWorkFlow_Activities_GetContextDataActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetContextDataActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.GetContextDataActivity';

    //保存自身
    serializeContext["dataKey"] = this.get_dataKey();
    serializeContext["dataContextLayer"] = this.get_dataContextLayer();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.GetContextDataActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_GetContextDataActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetContextDataActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetContextDataActivity, this, 'execute', [context]);

    var application = context.get_application();
    //根据指定的dataContextLayer获取数据，并存放于result之中
    var dataKey = this.get_dataKey();
    var dataContextLayer = this.get_dataContextLayer();
    var result = null;

    if (jsWorkFlow.DataContextLayer.auto === dataContextLayer) {
        //自动确定遍历的所在
        //数据加载以夫的context作为local的定义context，可以检索到private数据
        result = application.getData(dataKey, 1);
    }
    else {
        //
        var dataContext = null;
        if (jsWorkFlow.DataContextLayer.local === dataContextLayer) {
            //当前层是GetContextDataActivity，访问上一层
            dataContext = application.internalContextAt(1);

        }
        else if (jsWorkFlow.DataContextLayer.application === dataContextLayer) {
            //app层级
            dataContext = application.get_appContext();
        }
        else if (jsWorkFlow.DataContextLayer.global === dataContextLayer) {
            //global层级
            dataContext = application.get_globalContext();
        }
        else {
            //LOG Here!
            throw jso.errorInvalidOperation("Unknown dataContextLayer value [" + dataContextLayer + "]!");
        }

        result = dataContext.getData(dataKey);
    }

    //设置运行结果到context的result
    context.set_result(result);

    //TODO:
    //    LOG noop message!

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.GetContextDataActivity.prototype = {
    _dataKey: null,
    _dataContextLayer: 0,
    dispose: jsWorkFlow_Activities_GetContextDataActivity$dispose,
    //property
    get_dataKey: jsWorkFlow_Activities_GetContextDataActivity$get_dataKey,
    set_dataKey: jsWorkFlow_Activities_GetContextDataActivity$set_dataKey,
    get_dataContextLayer: jsWorkFlow_Activities_GetContextDataActivity$get_dataContextLayer,
    set_dataContextLayer: jsWorkFlow_Activities_GetContextDataActivity$set_dataContextLayer,
    //method
    loadSerializeContext: jsWorkFlow_Activities_GetContextDataActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_GetContextDataActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_GetContextDataActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.GetContextDataActivity, 'jsWorkFlow.Activities.GetContextDataActivity'), 
    jsWorkFlow.Activity);



//////////////////////////////////////////////////////////////////////////////////////////
//SetContextDataActivity
//
//TO 开发者：
//    是设置ContextData的activity，用于设置数据。
//    和DefineContextDataActivity功能类似，但是不同于DefineContextDataActivity，SetContextDataActivity
//并不产生新的数据定义。如果设置成功，activity的结果是true；如果没有找到数据定义，返回false。
//    另外一个和DefineContextDataActivity不同的地方，SetContextDataActivity并不允许指定isPrivate，
//因为本身是数据的消费者。
//
jsWorkFlow.Activities.SetContextDataActivity = function jsWorkFlow_Activities_SetContextDataActivity(dataKey, dataValueActivity, dataContextLayer) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity execute!");


    jso.initializeBase(jsWorkFlow.Activities.SetContextDataActivity, this);

    if (typeof (dataContextLayer) === "undefined") {
        //默认数据存储在activity层级上
        dataContextLayer = jsWorkFlow.DataContextLayer.auto;
    }

    this._dataKey = dataKey;
    this._dataValueActivity = dataValueActivity;
    this._dataContextLayer = dataContextLayer;

    this._doEvalDataValueActivityCompleteHandler = jso.createDelegate(this, this.doEvalDataValueActivityCompleteHandler);

};

function jsWorkFlow_Activities_SetContextDataActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.SetContextDataActivity, this, 'dispose');

    this._dataKey = null;
    this._dataValueActivity = null;
}

function jsWorkFlow_Activities_SetContextDataActivity$get_dataKey() {
    return this._dataKey;
}

function jsWorkFlow_Activities_SetContextDataActivity$set_dataKey(value) {
    this._dataKey = value;
}

function jsWorkFlow_Activities_SetContextDataActivity$get_dataValueActivity() {
    return this._dataValueActivity;
}

function jsWorkFlow_Activities_SetContextDataActivity$set_dataValueActivity(value) {
    this._dataValueActivity = value;
}

function jsWorkFlow_Activities_SetContextDataActivity$get_dataContextLayer() {
    return this._dataContextLayer;
}

function jsWorkFlow_Activities_SetContextDataActivity$set_dataContextLayer(value) {
    this._dataContextLayer = value;
}

//activity的恢复
function jsWorkFlow_Activities_SetContextDataActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.SetContextDataActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.SetContextDataActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    //dataKey
    var dataKey = serializeContext["dataKey"];
    this.set_dataKey(dataKey);
    //dataValueActivity
    var dataValueActivitySC = serializeContext["dataValueActivity"];
    var dataValueActivity = $jwf.loadActivity(dataValueActivitySC);
    this.set_dataValueActivity(dataValueActivity);
    //dataContextLayer
    var dataContextLayer = serializeContext["dataContextLayer"];
    this.set_dataContextLayer(dataContextLayer);
    //isPrivate
    var isPrivate = serializeContext["isPrivate"];
}

//activity的序列化
function jsWorkFlow_Activities_SetContextDataActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.SetContextDataActivity';

    //保存自身
    //dataKey
    serializeContext["dataKey"] = this.get_dataKey();
    //dataValueActivity
    var dataValueActivity = this.get_dataValueActivity();
    serializeContext["dataValueActivity"] = $jwf.saveActivity(dataValueActivity);
    //dataContextLayer
    serializeContext["dataContextLayer"] = this.get_dataContextLayer();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.SetContextDataActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_SetContextDataActivity$doSetContextDataValue(application, value) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity doSetContextDataValue!");

    //使用给出的值来初始化contextData
    var dataContextLayer = this._dataContextLayer;
    var dataKey = this._dataKey;
    var result = false;

    //根据dataContextLayer来初始化数据项
    if (jsWorkFlow.DataContextLayer.auto === dataContextLayer) {
        //自动确定遍历的所在
        //数据加载以夫的context作为local的定义context，可以检索到private数据
        var dataItem = application.getDataItem(dataKey, 1);
        if (dataItem) {
            dataItem.value = value;
            result = true;
        }
    }
    else {
        if (jsWorkFlow.DataContextLayer.local === dataContextLayer) {
            //activity层级的数据定义，要跳过当前（SetContextDataActivity）层，将数据存放上一层之中
            dataContext = application.internalContextAt(1);
        }
        else if (jsWorkFlow.DataContextLayer.application === dataContextLayer) {
            //app层级
            dataContext = application.get_appContext();
        }
        else if (jsWorkFlow.DataContextLayer.global === dataContextLayer) {
            //global层级
            dataContext = application.get_globalContext();
        }
        else {
            //LOG Here!
            throw jso.errorInvalidOperation("Unknown dataContextLayer value [" + dataContextLayer + "]!");
        }

        var dataItem = dataContext.getDataItem(dataKey);

        if (dataItem) {
            dataItem.value = value;
            result = true;
        }
    }

    return result;
}

function jsWorkFlow_Activities_SetContextDataActivity$doEvalDataValueActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity doEvalDataValueActivityCompleteHandler!");

    //post complete事件
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var application = parentContext.get_application();

    //默认值是null
    var dataValue = null;
    var result = false;

    //cleanup old executor
    executor.remove_postComplete(this._doEvalDataValueActivityCompleteHandler);

    //取求值结果
    dataValue = context.get_result();

    //设置ContextData的值
    result = this.doSetContextDataValue(application, dataValue);

    //将parent的result设置为doSetContextDataValue的结果
    parentContext.set_result(result);

    //执行完毕，结束activity
    $jwf.endActivity(parentContext);
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_SetContextDataActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetContextDataActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.SetContextDataActivity, this, 'execute', [context]);

    //对dataValueActivity求值
    //将值放到dataKey之中

    //选择activity，并执行
    var activity = this._dataValueActivity;
    var application = context.get_application();
    var result = false;

    if (!activity) {
        //空的activity，认为是null值
        result = this.doSetContextDataValue(application, null);
        context.set_result(result);
        //执行完毕，结束activity
        $jwf.endActivity(context);
        return;
    }

    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalDataValueActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

jsWorkFlow.Activities.SetContextDataActivity.prototype = {
    _dataKey: null,
    _dataValueActivity: null,
    _dataContextLayer: 0,
    dispose: jsWorkFlow_Activities_SetContextDataActivity$dispose,
    //property
    get_dataKey: jsWorkFlow_Activities_SetContextDataActivity$get_dataKey,
    set_dataKey: jsWorkFlow_Activities_SetContextDataActivity$set_dataKey,
    get_dataValueActivity: jsWorkFlow_Activities_SetContextDataActivity$get_dataValueActivity,
    set_dataValueActivity: jsWorkFlow_Activities_SetContextDataActivity$set_dataValueActivity,
    get_dataContextLayer: jsWorkFlow_Activities_SetContextDataActivity$get_dataContextLayer,
    set_dataContextLayer: jsWorkFlow_Activities_SetContextDataActivity$set_dataContextLayer,
    //method
    loadSerializeContext: jsWorkFlow_Activities_SetContextDataActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_SetContextDataActivity$saveSerializeContext,
    doSetContextDataValue: jsWorkFlow_Activities_SetContextDataActivity$doSetContextDataValue,
    doEvalDataValueActivityCompleteHandler: jsWorkFlow_Activities_SetContextDataActivity$doEvalDataValueActivityCompleteHandler,
    execute: jsWorkFlow_Activities_SetContextDataActivity$execute

};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.SetContextDataActivity, 'jsWorkFlow.Activities.SetContextDataActivity'), 
    jsWorkFlow.Activity);



//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//ParallelActivity
//
//TO 开发者：
//    提供同时执行activity的运行环境。在javascript中实际上是不存在并行的运行环境，不过存在异步
//的处理模式，ParallelActivity并不担保会让所有的activity并行执行，只是会尽量提前activity的执行
//时机。
//
jsWorkFlow.Activities.ParallelActivity = function jsWorkFlow_Activities_ParallelActivity(activities) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.ParallelActivity, this);

    this._activities = activities;
    this._doExecuteItemCompleteHandler = jso.createDelegate(this, this.doExecuteItemCompleteHandler);
};

function jsWorkFlow_Activities_ParallelActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.ParallelActivity, this, 'dispose');
}

function jsWorkFlow_Activities_ParallelActivity$get_activities() {
    return this._activities;
}

function jsWorkFlow_Activities_ParallelActivity$set_activities(activities) {
    if(!activities) {
        activities = [];
    }

    this._activities = activities;
}

function jsWorkFlow_Activities_ParallelActivity$addActivity(activity) {
    var activities = this.get_activities();
    jso.arrayAdd(activities, activity);
}

function jsWorkFlow_Activities_ParallelActivity$removeActivity(activity) {
    var activities = this.get_activities();
    Array.remove(activities, activity);
}

function jsWorkFlow_Activities_ParallelActivity$removeActivityAt(index) {
    var activities = this.get_activities();
    Array.removeAt(activities, index);
}

function jsWorkFlow_Activities_ParallelActivity$clearActivity() {
    var activities = this.get_activities();
    Array.clear(activities);
}

//activity的恢复
function jsWorkFlow_Activities_ParallelActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.ParallelActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    //恢复自身
    jso.callBaseMethod(jsWorkFlow.Activities.ParallelActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    var activities = [];
    var activitiesSC = serializeContext["activities"];

    if (activitiesSC && (activitiesSC.length > 0)) {
        for (var i = 0, ilen = activitiesSC.length; i < ilen; i++) {
            var item = $jwf.loadActivity(activitiesSC[i]);
            jso.arrayAdd(activities, item);
        }
    }

    //恢复activities
    this.set_activities(activities);

}

//activity的序列化
function jsWorkFlow_Activities_ParallelActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.ParallelActivity';

    var activities = this.get_activities();
    var activitiesSC = [];

    if (activities && (activities.length > 0)) {
        for (var i = 0, ilen = activities.length; i < ilen; i++) {
            var itemSC = $jwf.saveActivity(activities[i]);
            jso.arrayAdd(activitiesSC, itemSC);
        }
    }

    //保存activities
    serializeContext["activities"] = activitiesSC; 

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.ParallelActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_ParallelActivity$doExecuteItem(context, activity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity doExecuteItem!");

    
    //取activity
    if (!activity) {
        context.completeCount++;
        if (context.completeCount == context.totalActivity) {
            $jwf.endActivity(context);
        }
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecuteItemCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_ParallelActivity$doExecuteItemCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity doExecuteItemCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var condition = context.get_result();

    parentContext.completeCount++;
    if (parentContext.completeCount == parentContext.totalActivity) {
        $jwf.endActivity(parentContext);
    }
    return;

}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_ParallelActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ParallelActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.ParallelActivity, this, 'execute', [context]);

    //因为每个activity都是通过job来执行，本身就是异步的。循环执行activityExecutor就可以。

    var activities = this.get_activities();

    if(!activities || (activities.length == 0)) {
        //空activity列表，可以直接结束了
        $jwf.endActivity(context);
        return;
    }

    context.totalActivity = activities.length;
    context.completeCount = 0;

    for(var i = 0,ilen=activities.length;i<ilen;i++) {
        var activity = activities[i];
        this.doExecuteItem(context,activity); 
    }
}

jsWorkFlow.Activities.ParallelActivity.prototype = {
    _activities: null,
    _doExecuteItemCompleteHandler: null,
    dispose: jsWorkFlow_Activities_ParallelActivity$dispose,
    //property
    get_activities: jsWorkFlow_Activities_ParallelActivity$get_activities,
    set_activities: jsWorkFlow_Activities_ParallelActivity$set_activities,
    addActivity: jsWorkFlow_Activities_ParallelActivity$addActivity,
    removeActivity: jsWorkFlow_Activities_ParallelActivity$removeActivity,
    removeActivityAt: jsWorkFlow_Activities_ParallelActivity$removeActivityAt,
    clearActivity: jsWorkFlow_Activities_ParallelActivity$clearActivity,

    //method
    loadSerializeContext: jsWorkFlow_Activities_ParallelActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_ParallelActivity$saveSerializeContext,
    doExecuteItem: jsWorkFlow_Activities_ParallelActivity$doExecuteItem,
    doExecuteItemCompleteHandler: jsWorkFlow_Activities_ParallelActivity$doExecuteItemCompleteHandler,
    execute: jsWorkFlow_Activities_ParallelActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.ParallelActivity, 'jsWorkFlow.Activities.ParallelActivity'), 
    jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');



//////////////////////////////////////////////////////////////////////////////////////////
//SequenceActivity，包装基本的activity队列，按次序执行
//
//TO 开发者：
//    是一个activity的容器，允许将多个activity放到SequenceActivity中按加入的次序顺序执行。
//SequenceActivity将其执行的最后一个activity作为自己的返回值携带回来。
//
jsWorkFlow.Activities.SequenceActivity = function jsWorkFlow_Activities_SequenceActivity(activities) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.SequenceActivity, this);

    if (!activities) {
        activities = [];
    }

    this._activities = activities;
    this._doActivityCompleteHandler = jso.createDelegate(this, this.doActivityCompleteHandler);
};

function jsWorkFlow_Activities_SequenceActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.SequenceActivity, this, 'dispose');

    this._activities = null;
    this._doActivityCompleteHandler = null;
}

function jsWorkFlow_Activities_SequenceActivity$get_activities() {
    return this._activities;
}

function jsWorkFlow_Activities_SequenceActivity$set_activities(value) {
    this._activities = value;
}

//activity的恢复
function jsWorkFlow_Activities_SequenceActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.SequenceActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.SequenceActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var activitiesSC = serializeContext['activities'];
    var activities = [];

    if (activitiesSC && (activitiesSC.length > 0)) {
        for (var i = 0, ilen = activitiesSC.length; i < ilen; i++) {
            var activity = $jwf.loadActivity(activitiesSC[i]);
            jso.arrayAdd(activities, activity);
        }
    }

    this.set_activities(activities);
}

//activity的序列化
function jsWorkFlow_Activities_SequenceActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.SequenceActivity';

    //保存自身
    var activitiesSC = [];
    var activities = this.get_activities();

    if (activities && (activities.length > 0)) {
        for (var i = 0, ilen = activities.length; i < ilen; i++) {
            var activitySC = $jwf.saveActivity(activities[i]);
            jso.arrayAdd(activitiesSC, activitySC); 
        }
    }

    serializeContext['activities'] = activitiesSC;

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.SequenceActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_SequenceActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.SequenceActivity, this, 'execute', [context]);

    //从索引0开始依次执行
    this.doExecActivity(context, 0);
}

function jsWorkFlow_Activities_SequenceActivity$addActivity(activity) {
    jso.arrayAdd(this._activities, activity);
}

function jsWorkFlow_Activities_SequenceActivity$doExecActivity(context, index) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity doExecActivity!");

    if (index >= this._activities.length) {
        //OK，所有activity都执行完毕了，结束当前的活动
        $jwf.endActivity(context);
        return;
    }

    //选择activity，并执行
    var application = context.get_application();
    var activity = this._activities[index];
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    //将上下文的数据存放在activityExecutor之中
    activityExecutor.currentIndex = index;

    activityExecutor.add_postComplete(this._doActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_SequenceActivity$doActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SequenceActivity doActivityCompleteHandler!");


    //post complete事件
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var currentIndex = executor.currentIndex;

    //cleanup old executor
    executor.remove_postComplete(this._doActivityCompleteHandler);

    //set last context's result as current context's result
    var lastResult = context.get_result();
    parentContext.set_result(lastResult);

    //执行下一个迭代
    currentIndex++;
    this.doExecActivity(parentContext, currentIndex);
}

jsWorkFlow.Activities.SequenceActivity.prototype = {
    _activities: null,
    _doActivityCompleteHandler: null,
    dispose: jsWorkFlow_Activities_SequenceActivity$dispose,
    //property
    get_activities: jsWorkFlow_Activities_SequenceActivity$get_activities,
    set_activities: jsWorkFlow_Activities_SequenceActivity$set_activities,
    //method
    loadSerializeContext: jsWorkFlow_Activities_SequenceActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_SequenceActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_SequenceActivity$execute,
    addActivity: jsWorkFlow_Activities_SequenceActivity$addActivity,
    //执行activity，并挂接ActivityExecutor的postComplete事件
    doExecActivity: jsWorkFlow_Activities_SequenceActivity$doExecActivity,
    //ActivityExecutor的postComplete的handler
    doActivityCompleteHandler: jsWorkFlow_Activities_SequenceActivity$doActivityCompleteHandler

};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.SequenceActivity, 'jsWorkFlow.Activities.SequenceActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//StateMachineActivity
//
//TO 开发者：
//    提供一个状态机。允许注册有限个状态，并可以驱动当前的状态机变更状态，通过状态的变更
//驱动关联的activity的执行，并通过执行的activity继续驱动状态变更；
//    StateMachineActivity的目标不是设计状态迁移图，而是提供有限状态机的执行载体，并允许
//将整个状态机作为一个活动放置到workflow之中。
//    允许注册有限个状态，none、start、end、error是系统预定义的状态，其它的状态自己指定。
//状态定义完整了就可以启动状态机了。
//    状态机启动就是start状态，结束的状态可以是end或error，通过将状态设置成end或error来结束
//状态机的执行。（error状态比较特殊，可以从error状态恢复，也可以直接结束状态机）
//    在外部通过将activity注册到关注的目标状态或状态迁移来执行activity。
//
jsWorkFlow.Activities.StateMachineActivity = function jsWorkFlow_Activities_StateMachineActivity(executeActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity create!");


    jso.initializeBase(jsWorkFlow.Activities.StateMachineActivity, this);

    this._executeActivity = executeActivity;
};

function jsWorkFlow_Activities_StateMachineActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.StateMachineActivity, this, 'dispose');
    this._executeActivity = null;
}

function jsWorkFlow_Activities_StateMachineActivity$get_executeActivity() {
    return this._executeActivity;
}

function jsWorkFlow_Activities_StateMachineActivity$set_executeActivity(value) {
    this._executeActivity = value;
}

function jsWorkFlow_Activities_StateMachineActivity$notifyStateChanged(context, oldState, curState) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity notifyStateChanged!");

    log.debug("oldState:[" + oldState + "]!");
    log.debug("curState:[" + curState + "]!");

    //先执行基类的notifyStateChanged
    jso.callBaseMethod(jsWorkFlow.Activities.StateMachineActivity, this, 'notifyStateChanged', [context, oldState, curState]);

    //如果状态不是end or error，通知状态变更，执行executeActivity
    if ((jsWorkFlow.ActivityState.end === curState) || (jsWorkFlow.ActivityState.error === curState)) {
        return;
    }

    //如果executeActivity不存在，意味着状态机是空的，没有必要等待
    executeActivity = this.get_executeActivity();

    if (!executeActivity) {
        //结束activity
        $jwf.endActivity(context);
        return;
    }

    //执行executeActivity，一般在executeActivity的执行过程中，会驱动状态发送变更，再次进入notifyStateChanged，或结束状态机的执行。
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, executeActivity, context);
    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

//activity的恢复
function jsWorkFlow_Activities_StateMachineActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.StateMachineActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.StateMachineActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var executeActivitySC = serializeContext["executeActivity"];
    var executeActivity = $jwf.loadActivity(executeActivitySC);
    this.set_executeActivity(executeActivity);

}

//activity的序列化
function jsWorkFlow_Activities_StateMachineActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.StateMachineActivity';

    //保存自身
    var executeActivity = this.get_executeActivity();
    serializeContext["executeActivity"] = $jwf.saveActivity(executeActivity);

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.StateMachineActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_StateMachineActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.StateMachineActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.StateMachineActivity, this, 'execute', [context]);

    //TODO:
    //    LOG noop message!

    //StateMachine的状态变更是通过活动来驱动的，也就是说，通过外部对ActivityContext中的状态的变更来驱动状态机的执行，
    //状态机的结束也是通过状态驱动来完成的。
    
    //状态机的状态变更驱动Activity的执行，Activity通过其Context获取状态机的Context，并设置状态

    //执行完毕，结束activity
    //$jwf.endActivity(context);
}

jsWorkFlow.Activities.StateMachineActivity.prototype = {
    _executeActivity: null,
    dispose: jsWorkFlow_Activities_StateMachineActivity$dispose,
    //property
    get_executeActivity: jsWorkFlow_Activities_StateMachineActivity$get_executeActivity,
    set_executeActivity: jsWorkFlow_Activities_StateMachineActivity$set_executeActivity,
    //method
    notifyStateChanged: jsWorkFlow_Activities_StateMachineActivity$notifyStateChanged,
    execute: jsWorkFlow_Activities_StateMachineActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.StateMachineActivity, 'jsWorkFlow.Activities.StateMachineActivity'), 
    jsWorkFlow.Activity);

//根据context搜索StateMachine的context
function $jwf$sm$findStateMachineContext(context) {
    var retval = null;

    if (!context) {
        return retval;
    }

    var app = context.get_application();

    if (!app) {
        return retval;
    }

    //根据contextStack进行搜索
    var contextStack = app._contextStack;

    for (var i = 0, ilen = contextStack.length; i < ilen; i++) {
        var contextItem = contextStack[ilen - i - 1];

        var contextItemAct = contextItem.get_activity();

        //
        if (jso.isInstanceOfType(contextItemAct, jsWorkFlow.Activities.StateMachineActivity)) {
            //OK, find it!
            return contextItem;
        }
    }

    return retval;
}

//////////////////////////////////////////////////////////////////////////////////////////
//GetStateMachineStateActivity
//
//TO 开发者：
//    根据所在的状态机，获取状态机的状态，并将结果放置于Context的result之中
//
jsWorkFlow.Activities.GetStateMachineStateActivity = function jsWorkFlow_Activities_GetStateMachineStateActivity() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetStateMachineStateActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.GetStateMachineStateActivity, this);

};

function jsWorkFlow_Activities_GetStateMachineStateActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetStateMachineStateActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetStateMachineStateActivity, this, 'dispose');
}


//activity的恢复
function jsWorkFlow_Activities_GetStateMachineStateActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetStateMachineStateActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.GetStateMachineStateActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.GetStateMachineStateActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身

}

//activity的序列化
function jsWorkFlow_Activities_GetStateMachineStateActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetStateMachineStateActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.GetStateMachineStateActivity';

    //保存自身

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.GetStateMachineStateActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_GetStateMachineStateActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetStateMachineStateActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetStateMachineStateActivity, this, 'execute', [context]);

    var smContext = $jwf$sm$findStateMachineContext(context);
    var smState = jsWorkFlow.ActivityState.none;

    if (smContext) {
        smState = smContext.get_activityState();
    }

    //将状态机的状态传递到context的result中，然后结束
    context.set_result(smState);

    //TODO:
    //    LOG noop message!

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.GetStateMachineStateActivity.prototype = {
    dispose: jsWorkFlow_Activities_GetStateMachineStateActivity$dispose,
    //property
    //method
    loadSerializeContext: jsWorkFlow_Activities_GetStateMachineStateActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_GetStateMachineStateActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_GetStateMachineStateActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.GetStateMachineStateActivity, 'jsWorkFlow.Activities.GetStateMachineStateActivity'), 
    jsWorkFlow.Activity);



//////////////////////////////////////////////////////////////////////////////////////////
//SetStateMachineStateActivity
//
//TO 开发者：
//    根据所在的状态机，设置状态机的状态，并将状态机之前的状态放置于Context的result之中。
//结束状态机通过设置状态为：jsWorkFlow.ActivityState.end
//
jsWorkFlow.Activities.SetStateMachineStateActivity = function jsWorkFlow_Activities_SetStateMachineStateActivity(stateActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.SetStateMachineStateActivity, this);

    this._stateActivity = stateActivity;
    this._doStateActivityCompleteHandler = jso.createDelegate(this, this.doStateActivityCompleteHandler);

};

function jsWorkFlow_Activities_SetStateMachineStateActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.SetStateMachineStateActivity, this, 'dispose');

    this._stateActivity = null;
    this._doStateActivityCompleteHandler = null;
}

//需设置的状态，通过activity的执行结果来获取
function jsWorkFlow_Activities_SetStateMachineStateActivity$get_stateActivity() {
    return this._stateActivity;
}

function jsWorkFlow_Activities_SetStateMachineStateActivity$set_stateActivity(value) {
    this._stateActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_SetStateMachineStateActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== this.getType().getName()) {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.SetStateMachineStateActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var stateActivitySC = serializeContext["stateActivity"];
    var stateActivity = $jwf.loadActivity(stateActivitySC);
    this.set_stateActivity(stateActivity);

}

//activity的序列化
function jsWorkFlow_Activities_SetStateMachineStateActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.SetStateMachineStateActivity';

    //保存自身
    var stateActivity = this.get_stateActivity();
    serializeContext["stateActivity"] = $jwf.saveActivity(stateActivity);

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.SetStateMachineStateActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_SetStateMachineStateActivity$doStateActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity doStateActivityCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //取执行结果
    var curState = context.get_result();

    //获取StateMachine的context
    var smContext = $jwf$sm$findStateMachineContext(context);
    var oldState = jsWorkFlow.ActivityState.none;

    if (smContext) {
        oldState = smContext.get_activityState();
        smContext.set_activityState(curState);
        //将状态机的旧状态传递到parentContext的result中
        parentContext.set_result(oldState);
    }
    else {
        //不可能发生的情况！
        //LOG ERROR!
    }

    //结束parentContext的执行
    $jwf.endActivity(parentContext);
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_SetStateMachineStateActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SetStateMachineStateActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.SetStateMachineStateActivity, this, 'execute', [context]);

    //如果存在stateActivity，执行其，并获取最新的状态
    var stateActivity = this.get_stateActivity();
    if (!stateActivity) {
        //执行完毕，结束activity
        $jwf.endActivity(context);
        return;
    }

    //执行stateActivity获取新状态
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, stateActivity, context);

    activityExecutor.add_postComplete(this._doStateActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();


    //TODO:
    //    LOG noop message!

    ////执行完毕，结束activity
    //$jwf.endActivity(context);
}

jsWorkFlow.Activities.SetStateMachineStateActivity.prototype = {
    _stateActivity: null,
    _doStateActivityCompleteHandler: null,
    dispose: jsWorkFlow_Activities_SetStateMachineStateActivity$dispose,
    //property
    get_stateActivity: jsWorkFlow_Activities_SetStateMachineStateActivity$get_stateActivity,
    set_stateActivity: jsWorkFlow_Activities_SetStateMachineStateActivity$set_stateActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_SetStateMachineStateActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_SetStateMachineStateActivity$saveSerializeContext,
    doStateActivityCompleteHandler: jsWorkFlow_Activities_SetStateMachineStateActivity$doStateActivityCompleteHandler,
    execute: jsWorkFlow_Activities_SetStateMachineStateActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.SetStateMachineStateActivity, 'jsWorkFlow.Activities.SetStateMachineStateActivity'), 
    jsWorkFlow.Activity);


//////////////////////////////////////////////////////////////////////////////////////////
//EndStateMachineActivity
//
//TO 开发者：
//    用于结束一个StateMachine的执行，使其进入结束状态。
//
jsWorkFlow.Activities.EndStateMachineActivity = function jsWorkFlow_Activities_EndStateMachineActivity() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EndStateMachineActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.EndStateMachineActivity, this);

};

function jsWorkFlow_Activities_EndStateMachineActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EndStateMachineActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.EndStateMachineActivity, this, 'dispose');
}


//activity的恢复
function jsWorkFlow_Activities_EndStateMachineActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EndStateMachineActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.EndStateMachineActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.EndStateMachineActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身

}

//activity的序列化
function jsWorkFlow_Activities_EndStateMachineActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EndStateMachineActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.EndStateMachineActivity';

    //保存自身

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.EndStateMachineActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_EndStateMachineActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.EndStateMachineActivity execute!");


    jso.callBaseMethod(jsWorkFlow.Activities.EndStateMachineActivity, this, 'execute', [context]);

    //获取StateMachine的context
    var smContext = $jwf$sm$findStateMachineContext(context);
    var oldState = jsWorkFlow.ActivityState.none;

    if (smContext) {
        oldState = smContext.get_activityState();
        //结束smContext的执行
        $jwf.endActivity(smContext);
        //将oldState作为activity的执行结果
        context.set_result(oldState);
    }
    else {
        //不可能发生的情况！
        //LOG ERROR!
    }

    //TODO:
    //    LOG noop message!

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.EndStateMachineActivity.prototype = {
    dispose: jsWorkFlow_Activities_EndStateMachineActivity$dispose,
    //property
    //method
    loadSerializeContext: jsWorkFlow_Activities_EndStateMachineActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_EndStateMachineActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_EndStateMachineActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.EndStateMachineActivity, 'jsWorkFlow.Activities.EndStateMachineActivity'), 
    jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//SwitchActivity
//
//TO 开发者：
//    SwitchActivity用于处理多路分派的情况。可以处理Condition，并和特定的Activity的值来比较，
//并执行匹配的分支。
//    elseActivity对应没有匹配的Case的情况，这种情况下执行elseActivity。
//    allCase 是"key"/"value"字典组成的数组，这是规范的定义方式，不允许修改。
//
jsWorkFlow.Activities.SwitchActivity = function jsWorkFlow_Activities_SwitchActivity(conditionActivity, elseActivity, allCase) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.SwitchActivity, this);

    this._conditionActivity = conditionActivity ? conditionActivity : null;
    this._elseActivity = elseActivity ? elseActivity : null;
    this._allCase = allCase ? allCase : [];

    this._doEvalConditionCompleteHandler = jso.createDelegate(this, this.doEvalConditionCompleteHandler);
    this._doEvalCaseConditionCompleteHandler = jso.createDelegate(this, this.doEvalCaseConditionCompleteHandler);
    this._doExecuteCaseBodyCompleteHandler = jso.createDelegate(this, this.doExecuteCaseBodyCompleteHandler);
    this._doExecuteElseCaseCompleteHandler = jso.createDelegate(this, this.doExecuteElseCaseCompleteHandler);

};

function jsWorkFlow_Activities_SwitchActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity dispose!");

    this._conditionActivity = null;
    this._elseActivity = null;
    this._allCase = [];

    jso.callBaseMethod(jsWorkFlow.Activities.SwitchActivity, this, 'dispose');
}


function jsWorkFlow_Activities_SwitchActivity$get_conditionActivity() {
    return this._conditionActivity;
}

function jsWorkFlow_Activities_SwitchActivity$set_conditionActivity(value) {
    this._conditionActivity = value;
}

function jsWorkFlow_Activities_SwitchActivity$get_elseActivity() {
    return this._elseActivity;
}

function jsWorkFlow_Activities_SwitchActivity$set_elseActivity(value) {
    this._elseActivity = value;
}

function jsWorkFlow_Activities_SwitchActivity$get_allCase() {
    return this._allCase;
}

function jsWorkFlow_Activities_SwitchActivity$set_allCase(value) {
    this._allCase = value;
}


//activity的恢复
function jsWorkFlow_Activities_SwitchActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.SwitchActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.SwitchActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var conditionActivitySC = serializeContext["conditionActivity"];
    var conditionActivity = $jwf.loadActivity(conditionActivitySC);
    this.set_conditionActivity(conditionActivity);

    var elseActivitySC = serializeContext["elseActivity"];
    var elseActivity = $jwf.loadActivity(elseActivitySC);
    this.set_elseActivity(elseActivity);

    var allCase = [];
    var allCaseSC = serializeContext['allCase'];
    if (allCaseSC && (allCaseSC.length > 0)) {
        for (var i = 0, ilen = allCaseSC.length; i < ilen; i++) {
            var caseItemSC = allCaseSC[i];
            var caseItem = {};

            caseItem["key"] = $jwf.loadActivity(caseItemSC["key"]);
            caseItem["value"] = $jwf.loadActivity(caseItemSC["value"]);

            jso.arrayAdd(allCase, caseItem);
        }
    }

    this.set_allCase(allCase);
}

//activity的序列化
function jsWorkFlow_Activities_SwitchActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.SwitchActivity';

    //保存自身
    var conditionActivity = this.get_conditionActivity();
    serializeContext["conditionActivity"] = $jwf.saveActivity(conditionActivity);

    var elseActivity = this.get_elseActivity();
    serializeContext["elseActivity"] = $jwf.saveActivity(elseActivity);

    var allCase = this.get_allCase();
    var allCaseSC = [];
    if (allCase && (allCase.length > 0)) {
        for (var i = 0, ilen = allCase.length; i < ilen; i++) {
            var caseItem = allCase[i];
            var caseItemSC = {};

            caseItemSC["key"] = $jwf.saveActivity(caseItem["key"]);
            caseItemSC["value"] = $jwf.saveActivity(caseItem["value"]);

            jso.arrayAdd(allCaseSC, caseItemSC);
        }
    }

    serializeContext['allCase'] = allCaseSC;

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.SwitchActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_SwitchActivity$addCase(caseConditionActivity, caseBodyActivity) {
    var caseItem = {};
    caseItem["key"] = caseConditionActivity;
    caseItem["value"] = caseBodyActivity;

    this.addCaseItem(caseItem);
}

function jsWorkFlow_Activities_SwitchActivity$addCaseItem(caseItem) {
    var allCase = this.get_allCase();
    jso.arrayAdd(allCase, caseItem);
}

function jsWorkFlow_Activities_SwitchActivity$getCaseCount() {
    var allCase = this.get_allCase();
    return allCase.length;
}

function jsWorkFlow_Activities_SwitchActivity$getCaseItemByIndex(index) {
    var allCase = this.get_allCase();
    var caseItem = allCase[index];

    return caseItem;
}

function jsWorkFlow_Activities_SwitchActivity$removeCaseItemByIndex(index) {
    var allCase = this.get_allCase();
    var caseItem = allCase[index];
    Array.removeAt(index);

    return caseItem;
}

function jsWorkFlow_Activities_SwitchActivity$clearAllCase() {
    var allCase = this.get_allCase();

    this.set_allCase([]);
    return allCase;
}

//执行switch的条件，并返回值
function jsWorkFlow_Activities_SwitchActivity$doEvalCondition(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doEvalCondition!");


    //如果没有设置条件，取值为null
    var activity = this._conditionActivity;

    if (!activity) {
        //使用null来驱动doEvalCaseCondition执行
        this.doEvalCaseCondition(context, null, 0);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    //将上下文的数据存放在activityExecutor之中
    activityExecutor.parentContext = context;

    activityExecutor.add_postComplete(this._doEvalConditionCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_SwitchActivity$doEvalConditionCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doEvalConditionCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var condition = context.get_result();

    log.debug("condition is:[" + condition + "]");

    //将condition传递给doEvalCaseCondition继续执行
    this.doEvalCaseCondition(parentContext, condition, 0);
}

function jsWorkFlow_Activities_SwitchActivity$doEvalCaseCondition(context, condition, index) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doEvalCaseCondition!");
    log.debug("index is:[" + index + "]");

    var allCase = this.get_allCase();

    if (index >= allCase.length) {
        //OK，已经执行完毕所有的CASE
        //准备执行elseActivity
        this.doExecuteElseCase(context);
        return;
    }

    //取caseItem，准备匹配条件，执行caseBody
    var caseItem = this.getCaseItemByIndex(index);
    

    //如果没有设置条件，取值为null
    var activity = caseItem["key"];

    if (!activity) {
        if (condition === null) {
            //设置匹配的标记
            context.runCase = true;
            //执行对应的activity
            this.doExecuteCaseBody(context, index);
        }
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    //将上下文的数据存放在activityExecutor之中
    activityExecutor.condition = condition;
    activityExecutor.currentIndex = index;

    activityExecutor.add_postComplete(this._doEvalCaseConditionCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_SwitchActivity$doEvalCaseConditionCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doEvalCaseConditionCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var condition = executor.condition;
    var index = executor.currentIndex;

    //从context取执行结果
    var caseCondition = context.get_result();
    log.debug("caseCondition is:[" + caseCondition + "]");

    if (condition === caseCondition) {
        //有匹配的case, 设置runCase标记
        parentContext.runCase = true;
        //执行对应的Body
        this.doExecuteCaseBody(parentContext, index);
        return;
    }

    //没有匹配，递增index并评估下一个Case条件
    this.doEvalCaseCondition(parentContext, condition, index+1);
}


function jsWorkFlow_Activities_SwitchActivity$doExecuteCaseBody(context, index) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doExecuteCaseBody!");

    //执行对应的caseBody

    //取caseItem
    var caseItem = this.getCaseItemByIndex(index);

    var activity = caseItem["value"];

    if (!activity) {
        //如果不存在case body，则认为已经执行完毕，结束activity的执行
        $jwf.endActivity(context);

        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecuteCaseBodyCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_SwitchActivity$doExecuteCaseBodyCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doExecuteCaseBodyCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //执行完case body，可以结束switch activity的执行了
    $jwf.endActivity(parentContext);

}

function jsWorkFlow_Activities_SwitchActivity$doExecuteElseCase(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doExecuteElseCase!");

    var activity = this._elseActivity;

    //检查是否需要执行elseAvtivity
    if (context.runCase || !activity) {
        //不需要，那么结束activity的执行
        $jwf.endActivity(context);
        return;
    }

    //执行对应的activity

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecuteElseCaseCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_SwitchActivity$doExecuteElseCaseCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity doExecuteElseCaseCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //执行完case body，可以结束switch activity的执行了
    $jwf.endActivity(parentContext);

}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_SwitchActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.SwitchActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.SwitchActivity, this, 'execute', [context]);

    //初始化是否执行case的标记
    context.runCase = false;

    this.doEvalCondition(context);
}

jsWorkFlow.Activities.SwitchActivity.prototype = {
    _conditionActivity: null,
    _elseActivity: null,
    _allCase: null,
    _doEvalConditionCompleteHandler: null,
    _doEvalCaseConditionCompleteHandler: null,
    _doExecuteCaseBodyCompleteHandler: null,
    _doExecuteElseCaseCompleteHandler: null,
    dispose: jsWorkFlow_Activities_SwitchActivity$dispose,
    //property
    get_conditionActivity: jsWorkFlow_Activities_SwitchActivity$get_conditionActivity,
    set_conditionActivity: jsWorkFlow_Activities_SwitchActivity$set_conditionActivity,
    get_elseActivity: jsWorkFlow_Activities_SwitchActivity$get_elseActivity,
    set_elseActivity: jsWorkFlow_Activities_SwitchActivity$set_elseActivity,
    get_allCase: jsWorkFlow_Activities_SwitchActivity$get_allCase,
    set_allCase: jsWorkFlow_Activities_SwitchActivity$set_allCase,

    //method
    loadSerializeContext: jsWorkFlow_Activities_SwitchActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_SwitchActivity$saveSerializeContext,
    addCase: jsWorkFlow_Activities_SwitchActivity$addCase,
    addCaseItem: jsWorkFlow_Activities_SwitchActivity$addCaseItem,
    getCaseCount: jsWorkFlow_Activities_SwitchActivity$getCaseCount,
    getCaseItemByIndex: jsWorkFlow_Activities_SwitchActivity$getCaseItemByIndex,
    removeCaseItemByIndex: jsWorkFlow_Activities_SwitchActivity$removeCaseItemByIndex,
    clearAllCase: jsWorkFlow_Activities_SwitchActivity$clearAllCase,
    //workflow method
    doEvalCondition: jsWorkFlow_Activities_SwitchActivity$doEvalCondition,
    doEvalConditionCompleteHandler: jsWorkFlow_Activities_SwitchActivity$doEvalConditionCompleteHandler,
    doEvalCaseCondition: jsWorkFlow_Activities_SwitchActivity$doEvalCaseCondition,
    doEvalCaseConditionCompleteHandler: jsWorkFlow_Activities_SwitchActivity$doEvalCaseConditionCompleteHandler,
    doExecuteCaseBody: jsWorkFlow_Activities_SwitchActivity$doExecuteCaseBody,
    doExecuteCaseBodyCompleteHandler: jsWorkFlow_Activities_SwitchActivity$doExecuteCaseBodyCompleteHandler,
    doExecuteElseCase: jsWorkFlow_Activities_SwitchActivity$doExecuteElseCase,
    doExecuteElseCaseCompleteHandler: jsWorkFlow_Activities_SwitchActivity$doExecuteElseCaseCompleteHandler,
    execute: jsWorkFlow_Activities_SwitchActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.SwitchActivity, 'jsWorkFlow.Activities.SwitchActivity'), 
    jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//WhileActivity
//
//TO 开发者：
//    WhileActivity提供一个循环的执行体，通过conditionActivity来控制执行的循环条件，
// bodyActivity提供循环的执行体。
//    conditionActivity和bodyActivity都可以为空。
//
jsWorkFlow.Activities.WhileActivity = function jsWorkFlow_Activities_WhileActivity(conditionActivity, bodyActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.WhileActivity, this);

    this.set_conditionActivity(conditionActivity);
    this.set_bodyActivity(bodyActivity);

    this._doEvalConditionCompleteHandler = jso.createDelegate(this, this.doEvalConditionCompleteHandler);
    this._doExecuteBodyCompleteHandler = jso.createDelegate(this, this.doExecuteBodyCompleteHandler);

};

function jsWorkFlow_Activities_WhileActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.WhileActivity, this, 'dispose');
}

function jsWorkFlow_Activities_WhileActivity$get_conditionActivity() {
    return this._conditionActivity;
}

function jsWorkFlow_Activities_WhileActivity$set_conditionActivity(value) {
    this._conditionActivity = value;
}

function jsWorkFlow_Activities_WhileActivity$get_bodyActivity() {
    return this._bodyActivity;
}

function jsWorkFlow_Activities_WhileActivity$set_bodyActivity(value) {
    this._bodyActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_WhileActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.WhileActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }

    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.WhileActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    this.set_conditionActivity($jwf.loadActivity(serializeContext['conditionActivity']));
    this.set_bodyActivity($jwf.loadActivity(serializeContext['bodyActivity']));
}

//activity的序列化
function jsWorkFlow_Activities_WhileActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.WhileActivity';

    //保存自身
    serializeContext['conditionActivity'] = $jwf.saveActivity(this.get_conditionActivity());
    serializeContext['bodyActivity'] = $jwf.saveActivity(this.get_bodyActivity());


    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.WhileActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_WhileActivity$doEvalCondition(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity doEvalCondition!");

    //如果没有设置条件，认为为false
    var activity = this._conditionActivity;

    if (!activity) {
        this.doExecuteBody(context, false);
        return;
    }

    //构造ActivityExecutor执行conditionActivity
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalConditionCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_WhileActivity$doEvalConditionCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity doEvalConditionCompleteHandler!");

    //根据condition的执行结果，来执行
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var condition = context.get_result();
    log.debug("condition is:[" + condition + "]");

    //将condition传递给doExecuteBody继续执行
    this.doExecuteBody(parentContext, condition);

}

function jsWorkFlow_Activities_WhileActivity$doExecuteBody(context, condition) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity doExecuteBody!");

    if (!condition) {
        //循环条件为false，结束while的执行
        $jwf.endActivity(context);
        return;
    }

    //true，执行bodyActivity
    var activity = this._bodyActivity;

    if (!activity) {
        //没有activity，跳过，回到while的条件检查
        this.doEvalCondition(context);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecuteBodyCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_WhileActivity$doExecuteBodyCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity doExecuteBodyCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //将activity的执行解结果作为自身的结果
    var result = context.get_result();
    parentContext.set_result(result);

    //继续回到条件的检查点执行
    this.doEvalCondition(parentContext);
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_WhileActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.WhileActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.WhileActivity, this, 'execute', [context]);

    //从条件检查开始执行
    this.doEvalCondition(context);

}

jsWorkFlow.Activities.WhileActivity.prototype = {
    _conditionActivity: null,
    _bodyActivity: null,
    _doEvalConditionCompleteHandler: null,
    _doExecuteBodyCompleteHandler: null,
    //
    dispose: jsWorkFlow_Activities_WhileActivity$dispose,
    //property
    get_conditionActivity: jsWorkFlow_Activities_WhileActivity$get_conditionActivity,
    set_conditionActivity: jsWorkFlow_Activities_WhileActivity$set_conditionActivity,
    get_bodyActivity: jsWorkFlow_Activities_WhileActivity$get_bodyActivity,
    set_bodyActivity: jsWorkFlow_Activities_WhileActivity$set_bodyActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_WhileActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_WhileActivity$saveSerializeContext,
    doEvalCondition: jsWorkFlow_Activities_WhileActivity$doEvalCondition,
    doEvalConditionCompleteHandler: jsWorkFlow_Activities_WhileActivity$doEvalConditionCompleteHandler,
    doExecuteBody: jsWorkFlow_Activities_WhileActivity$doExecuteBody,
    doExecuteBodyCompleteHandler: jsWorkFlow_Activities_WhileActivity$doExecuteBodyCompleteHandler,
    execute: jsWorkFlow_Activities_WhileActivity$execute
};

jso.registerClass(jso.setTypeName(jsWorkFlow.Activities.WhileActivity, 'jsWorkFlow.Activities.WhileActivity'), jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//LogicAndActivity
//
//TO 开发者：
//    一个执行AND操作的activity。
//    对于非bool值向bool转换的规则，同javascript默认处理方式。undefined，null，0和""作为false。
//    遵循短路算法的逻辑。如果lha是false，那么rha将不会被execute。
//    如果activity为null，作为false处理。
//
jsWorkFlow.Activities.LogicAndActivity = function jsWorkFlow_Activities_LogicAndActivity(leftHandActivity, rightHandActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.LogicAndActivity, this);
    this.set_leftHandActivity(leftHandActivity);
    this.set_rightHandActivity(rightHandActivity);

    this._doExecLhaCompleteHandler = jso.createDelegate(this, this.doExecLhaCompleteHandler);
    this._doExecRhaCompleteHandler = jso.createDelegate(this, this.doExecRhaCompleteHandler);
};

function jsWorkFlow_Activities_LogicAndActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicAndActivity, this, 'dispose');
}


function jsWorkFlow_Activities_LogicAndActivity$get_leftHandActivity() {
    return this._leftHandActivity;
}

function jsWorkFlow_Activities_LogicAndActivity$set_leftHandActivity(value) {
    this._leftHandActivity = value;
}

function jsWorkFlow_Activities_LogicAndActivity$get_rightHandActivity() {
    return this._rightHandActivity;
}

function jsWorkFlow_Activities_LogicAndActivity$set_rightHandActivity(value) {
    this._rightHandActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_LogicAndActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.LogicAndActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.LogicAndActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var leftHandActivitySC = serializeContext['leftHandActivity'];
    var leftHandActivity = $jwf.loadActivity(leftHandActivitySC);
    this.set_leftHandActivity(leftHandActivity);

    var rightHandActivitySC = serializeContext['rightHandActivity'];
    var rightHandActivity = $jwf.loadActivity(rightHandActivitySC);
    this.set_rightHandActivity(rightHandActivity);
}

//activity的序列化
function jsWorkFlow_Activities_LogicAndActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.LogicAndActivity';

    //保存自身
    serializeContext['leftHandActivity'] = $jwf.saveActivity(this.get_leftHandActivity());
    serializeContext['rightHandActivity'] = $jwf.saveActivity(this.get_rightHandActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.LogicAndActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_LogicAndActivity$doExecLha(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity doExecLha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var lhaActivity = this._leftHandActivity;

    if (!lhaActivity) {
        this.doExecRha(context, false);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, lhaActivity, context);

    activityExecutor.add_postComplete(this._doExecLhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_LogicAndActivity$doExecLhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity doExecLhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //获取activity的最后的执行结果，作为doExecRha的输入
    var lhaResult = context.get_result();

    //执行Rha
    this.doExecRha(parentContext, lhaResult);

    return;
}

function jsWorkFlow_Activities_LogicAndActivity$doExecRha(context, lhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity doExecRha!");


    //执行短路逻辑，如果为false，后面的不用执行了。
    if (!lhaResult) {
        //结束activity的执行，并设置执行结果
        context.set_result(lhaResult);
        $jwf.endActivity(context);
        return;
    }

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var rhaActivity = this._rightHandActivity;

    if (!rhaActivity) {
        //结束activity的执行，并设置执行结果
        context.set_result(false);
        $jwf.endActivity(context);
        return;
    }

    //调度执行rha
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, rhaActivity, context);

    activityExecutor.add_postComplete(this._doExecRhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_LogicAndActivity$doExecRhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity doExecRhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //将activity的最后的执行结果作为当前activity的执行结果
    //因为既然调度rha执行，说明lha的执行结果一定是true，可以忽略lha，直接写入rha的执行结果。
    var result = context.get_result();
    parentContext.set_result(!!result);

    //结束activity的执行
    $jwf.endActivity(parentContext);
    return;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_LogicAndActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicAndActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicAndActivity, this, 'execute', [context]);

    this.doExecLha(context);
}

jsWorkFlow.Activities.LogicAndActivity.prototype = {
    _leftHandActivity: null,
    _rightHandActivity: null,
    _doExecLhaCompleteHandler: null,
    _doExecRhaCompleteHandler: null,
    dispose: jsWorkFlow_Activities_LogicAndActivity$dispose,
    //property
    get_leftHandActivity: jsWorkFlow_Activities_LogicAndActivity$get_leftHandActivity,
    set_leftHandActivity: jsWorkFlow_Activities_LogicAndActivity$set_leftHandActivity,
    get_rightHandActivity: jsWorkFlow_Activities_LogicAndActivity$get_rightHandActivity,
    set_rightHandActivity: jsWorkFlow_Activities_LogicAndActivity$set_rightHandActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_LogicAndActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_LogicAndActivity$saveSerializeContext,
    doExecLha: jsWorkFlow_Activities_LogicAndActivity$doExecLha,
    doExecLhaCompleteHandler: jsWorkFlow_Activities_LogicAndActivity$doExecLhaCompleteHandler,
    doExecRha: jsWorkFlow_Activities_LogicAndActivity$doExecRha,
    doExecRhaCompleteHandler: jsWorkFlow_Activities_LogicAndActivity$doExecRhaCompleteHandler,
    execute: jsWorkFlow_Activities_LogicAndActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.LogicAndActivity, 'jsWorkFlow.Activities.LogicAndActivity'), 
    jsWorkFlow.Activity);


//////////////////////////////////////////////////////////////////////////////////////////
//LogicOrActivity
//
//TO 开发者：
//    一个执行OR操作的activity。
//    对于非bool值向bool转换的规则，同javascript默认处理方式。undefined，null，0和""作为false。
//    遵循短路算法的逻辑。如果lha是true，那么rha将不会被execute。
//    如果activity为null，作为false处理。
//
jsWorkFlow.Activities.LogicOrActivity = function jsWorkFlow_Activities_LogicOrActivity(leftHandActivity, rightHandActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.LogicOrActivity, this);
    this.set_leftHandActivity(leftHandActivity);
    this.set_rightHandActivity(rightHandActivity);

    this._doExecLhaCompleteHandler = jso.createDelegate(this, this.doExecLhaCompleteHandler);
    this._doExecRhaCompleteHandler = jso.createDelegate(this, this.doExecRhaCompleteHandler);
};

function jsWorkFlow_Activities_LogicOrActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicOrActivity, this, 'dispose');
}


function jsWorkFlow_Activities_LogicOrActivity$get_leftHandActivity() {
    return this._leftHandActivity;
}

function jsWorkFlow_Activities_LogicOrActivity$set_leftHandActivity(value) {
    this._leftHandActivity = value;
}

function jsWorkFlow_Activities_LogicOrActivity$get_rightHandActivity() {
    return this._rightHandActivity;
}

function jsWorkFlow_Activities_LogicOrActivity$set_rightHandActivity(value) {
    this._rightHandActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_LogicOrActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.LogicOrActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.LogicOrActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var leftHandActivitySC = serializeContext['leftHandActivity'];
    var leftHandActivity = $jwf.loadActivity(leftHandActivitySC);
    this.set_leftHandActivity(leftHandActivity);

    var rightHandActivitySC = serializeContext['rightHandActivity'];
    var rightHandActivity = $jwf.loadActivity(rightHandActivitySC);
    this.set_rightHandActivity(rightHandActivity);
}

//activity的序列化
function jsWorkFlow_Activities_LogicOrActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.LogicOrActivity';

    //保存自身
    serializeContext['leftHandActivity'] = $jwf.saveActivity(this.get_leftHandActivity());
    serializeContext['rightHandActivity'] = $jwf.saveActivity(this.get_rightHandActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.LogicOrActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_LogicOrActivity$doExecLha(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity doExecLha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var lhaActivity = this._leftHandActivity;

    if (!lhaActivity) {
        this.doExecRha(context, false);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, lhaActivity, context);

    activityExecutor.add_postComplete(this._doExecLhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_LogicOrActivity$doExecLhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity doExecLhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //获取activity的最后的执行结果，作为doExecRha的输入
    var lhaResult = context.get_result();

    //执行Rha
    this.doExecRha(parentContext, lhaResult);

    return;
}

function jsWorkFlow_Activities_LogicOrActivity$doExecRha(context, lhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity doExecRha!");


    //执行短路逻辑，如果为true，后面的不用执行了。
    if (lhaResult) {
        //结束activity的执行，并设置执行结果
        context.set_result(lhaResult);
        $jwf.endActivity(context);
        return;
    }

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var rhaActivity = this._rightHandActivity;

    if (!rhaActivity) {
        //结束activity的执行，并设置执行结果
        context.set_result(false);
        $jwf.endActivity(context);
        return;
    }

    //调度执行rha
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, rhaActivity, context);

    activityExecutor.add_postComplete(this._doExecRhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_LogicOrActivity$doExecRhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity doExecRhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //将activity的最后的执行结果作为当前activity的执行结果
    //因为既然调度rha执行，说明lha的执行结果一定是false，可以忽略lha，直接写入rha的执行结果。
    var result = context.get_result();
    parentContext.set_result(!!result);

    //结束activity的执行
    $jwf.endActivity(parentContext);
    return;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_LogicOrActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicOrActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicOrActivity, this, 'execute', [context]);

    this.doExecLha(context);
}

jsWorkFlow.Activities.LogicOrActivity.prototype = {
    _leftHandActivity: null,
    _rightHandActivity: null,
    _doExecLhaCompleteHandler: null,
    _doExecRhaCompleteHandler: null,
    dispose: jsWorkFlow_Activities_LogicOrActivity$dispose,
    //property
    get_leftHandActivity: jsWorkFlow_Activities_LogicOrActivity$get_leftHandActivity,
    set_leftHandActivity: jsWorkFlow_Activities_LogicOrActivity$set_leftHandActivity,
    get_rightHandActivity: jsWorkFlow_Activities_LogicOrActivity$get_rightHandActivity,
    set_rightHandActivity: jsWorkFlow_Activities_LogicOrActivity$set_rightHandActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_LogicOrActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_LogicOrActivity$saveSerializeContext,
    doExecLha: jsWorkFlow_Activities_LogicOrActivity$doExecLha,
    doExecLhaCompleteHandler: jsWorkFlow_Activities_LogicOrActivity$doExecLhaCompleteHandler,
    doExecRha: jsWorkFlow_Activities_LogicOrActivity$doExecRha,
    doExecRhaCompleteHandler: jsWorkFlow_Activities_LogicOrActivity$doExecRhaCompleteHandler,
    execute: jsWorkFlow_Activities_LogicOrActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.LogicOrActivity, 'jsWorkFlow.Activities.LogicOrActivity'), 
    jsWorkFlow.Activity);


//////////////////////////////////////////////////////////////////////////////////////////
//LogicXorActivity
//
//TO 开发者：
//    一个执行XOR操作的activity。
//    对于非bool值向bool转换的规则，同javascript默认处理方式。undefined，null，0和""作为false。
//    如果activity为null，作为false处理。
//
jsWorkFlow.Activities.LogicXorActivity = function jsWorkFlow_Activities_LogicXorActivity(leftHandActivity, rightHandActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.LogicXorActivity, this);
    this.set_leftHandActivity(leftHandActivity);
    this.set_rightHandActivity(rightHandActivity);

    this._doExecLhaCompleteHandler = jso.createDelegate(this, this.doExecLhaCompleteHandler);
    this._doExecRhaCompleteHandler = jso.createDelegate(this, this.doExecRhaCompleteHandler);
};

function jsWorkFlow_Activities_LogicXorActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicXorActivity, this, 'dispose');
}


function jsWorkFlow_Activities_LogicXorActivity$get_leftHandActivity() {
    return this._leftHandActivity;
}

function jsWorkFlow_Activities_LogicXorActivity$set_leftHandActivity(value) {
    this._leftHandActivity = value;
}

function jsWorkFlow_Activities_LogicXorActivity$get_rightHandActivity() {
    return this._rightHandActivity;
}

function jsWorkFlow_Activities_LogicXorActivity$set_rightHandActivity(value) {
    this._rightHandActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_LogicXorActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.LogicXorActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.LogicXorActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var leftHandActivitySC = serializeContext['leftHandActivity'];
    var leftHandActivity = $jwf.loadActivity(leftHandActivitySC);
    this.set_leftHandActivity(leftHandActivity);

    var rightHandActivitySC = serializeContext['rightHandActivity'];
    var rightHandActivity = $jwf.loadActivity(rightHandActivitySC);
    this.set_rightHandActivity(rightHandActivity);
}

//activity的序列化
function jsWorkFlow_Activities_LogicXorActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.LogicXorActivity';

    //保存自身
    serializeContext['leftHandActivity'] = $jwf.saveActivity(this.get_leftHandActivity());
    serializeContext['rightHandActivity'] = $jwf.saveActivity(this.get_rightHandActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.LogicXorActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_LogicXorActivity$doExecLha(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity doExecLha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var lhaActivity = this._leftHandActivity;

    if (!lhaActivity) {
        this.doExecRha(context, false);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, lhaActivity, context);

    activityExecutor.add_postComplete(this._doExecLhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_LogicXorActivity$doExecLhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity doExecLhaCompleteHandler!");


    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //获取activity的最后的执行结果，作为doExecRha的输入
    var lhaResult = context.get_result();

    //执行Rha
    this.doExecRha(parentContext, lhaResult);

    return;
}

function jsWorkFlow_Activities_LogicXorActivity$doEvalXor(lhaResult, rhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity doEvalXor!");


    var lhaBit = 0;
    var rhaBit = 0;

    if (!!lhaResult) {
        lhaBit = 1;
    }

    if (!!rhaResult) {
        rhaBit = 1;
    }

    var result = lhaBit ^ rhaBit;

    return !!result;

}

function jsWorkFlow_Activities_LogicXorActivity$doExecRha(context, lhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity doExecRha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var rhaActivity = this._rightHandActivity;

    if (!rhaActivity) {
        var result = this.doEvalXor(lhaResult, false);
        //结束activity的执行，并设置执行结果
        context.set_result(result);
        $jwf.endActivity(context);
        return;
    }

    //调度执行rha
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, rhaActivity, context);

    //将上下文的数据存放在activityExecutor之中
    activityExecutor.lhaResult = lhaResult;

    activityExecutor.add_postComplete(this._doExecRhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_LogicXorActivity$doExecRhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity doExecRhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var lhaResult = executor.lhaResult;

    //结束activity的执行，并设置执行结果
    var rhaResult = context.get_result();
    var result = this.doEvalXor(lhaResult, rhaResult);
    parentContext.set_result(!!result);

    //结束activity的执行
    $jwf.endActivity(parentContext);
    return;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_LogicXorActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicXorActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicXorActivity, this, 'execute', [context]);

    this.doExecLha(context);
}

jsWorkFlow.Activities.LogicXorActivity.prototype = {
    _leftHandActivity: null,
    _rightHandActivity: null,
    _doExecLhaCompleteHandler: null,
    _doExecRhaCompleteHandler: null,
    dispose: jsWorkFlow_Activities_LogicXorActivity$dispose,
    //property
    get_leftHandActivity: jsWorkFlow_Activities_LogicXorActivity$get_leftHandActivity,
    set_leftHandActivity: jsWorkFlow_Activities_LogicXorActivity$set_leftHandActivity,
    get_rightHandActivity: jsWorkFlow_Activities_LogicXorActivity$get_rightHandActivity,
    set_rightHandActivity: jsWorkFlow_Activities_LogicXorActivity$set_rightHandActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_LogicXorActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_LogicXorActivity$saveSerializeContext,
    doExecLha: jsWorkFlow_Activities_LogicXorActivity$doExecLha,
    doExecLhaCompleteHandler: jsWorkFlow_Activities_LogicXorActivity$doExecLhaCompleteHandler,
    doEvalXor: jsWorkFlow_Activities_LogicXorActivity$doEvalXor,
    doExecRha: jsWorkFlow_Activities_LogicXorActivity$doExecRha,
    doExecRhaCompleteHandler: jsWorkFlow_Activities_LogicXorActivity$doExecRhaCompleteHandler,
    execute: jsWorkFlow_Activities_LogicXorActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.LogicXorActivity, 'jsWorkFlow.Activities.LogicXorActivity'), 
    jsWorkFlow.Activity);


//////////////////////////////////////////////////////////////////////////////////////////
//LogicNotActivity
//
//TO 开发者：
//    一个执行NOT操作的activity。
//    对于非bool值向bool转换的规则，同javascript默认处理方式。undefined，null，0和""作为false。
//    如果activity为null，作为false处理。
//
jsWorkFlow.Activities.LogicNotActivity = function jsWorkFlow_Activities_LogicNotActivity(activity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.LogicNotActivity, this);
    this.set_activity(activity);

    this._doExecActivityCompleteHandler = jso.createDelegate(this, this.doExecActivityCompleteHandler);
};

function jsWorkFlow_Activities_LogicNotActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicNotActivity, this, 'dispose');
}


function jsWorkFlow_Activities_LogicNotActivity$get_activity() {
    return this._activity;
}

function jsWorkFlow_Activities_LogicNotActivity$set_activity(value) {
    this._activity = value;
}

//activity的恢复
function jsWorkFlow_Activities_LogicNotActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.LogicNotActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.LogicNotActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var activitySC = serializeContext['activity'];
    var activity = $jwf.loadActivity(activitySC);
    this.set_activity(activity);

}

//activity的序列化
function jsWorkFlow_Activities_LogicNotActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.LogicNotActivity';

    //保存自身
    serializeContext['activity'] = $jwf.saveActivity(this.get_activity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.LogicNotActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_LogicNotActivity$doExecActivity(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity doExecActivity!");


    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var activity = this._activity;

    if (!activity) {
        context.set_result(true);
        $jwf.endActivity(context);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doExecActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_LogicNotActivity$doExecActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity doExecActivityCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //获取activity的最后的执行结果
    var result = context.get_result();

    //结束activity的执行，并设置执行结果
    parentContext.set_result(!result);
    $jwf.endActivity(parentContext);
    return;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_LogicNotActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.LogicNotActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.LogicNotActivity, this, 'execute', [context]);

    this.doExecActivity(context);
}

jsWorkFlow.Activities.LogicNotActivity.prototype = {
    _activity: null,
    _doExecActivityCompleteHandler: null,
    dispose: jsWorkFlow_Activities_LogicNotActivity$dispose,
    //property
    get_activity: jsWorkFlow_Activities_LogicNotActivity$get_activity,
    set_activity: jsWorkFlow_Activities_LogicNotActivity$set_activity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_LogicNotActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_LogicNotActivity$saveSerializeContext,
    doExecActivity: jsWorkFlow_Activities_LogicNotActivity$doExecActivity,
    doExecActivityCompleteHandler: jsWorkFlow_Activities_LogicNotActivity$doExecActivityCompleteHandler,
    execute: jsWorkFlow_Activities_LogicNotActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.LogicNotActivity, 'jsWorkFlow.Activities.LogicNotActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//ConstActivity
//
//TO 开发者：
//    定义包装常量的activity，仅用于展示activity得到基本控制流程，可以当成activity的占位
//符使用。
//
jsWorkFlow.Activities.ConstActivity = function jsWorkFlow_Activities_ConstActivity(constValue) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ConstActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.ConstActivity, this);

    this.set_constValue(constValue);
};

function jsWorkFlow_Activities_ConstActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ConstActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.ConstActivity, this, 'dispose');
}

function jsWorkFlow_Activities_ConstActivity$get_constValue() {
    return this._constValue;
}

function jsWorkFlow_Activities_ConstActivity$set_constValue(value) {
    this._constValue = value;
}

//activity的恢复
function jsWorkFlow_Activities_ConstActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ConstActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.ConstActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.ConstActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var constValue = serializeContext["constValue"];
    this.set_constValue(constValue);
}

//activity的序列化
function jsWorkFlow_Activities_ConstActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ConstActivity saveSerializeContext!");


    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.ConstActivity';

    //保存自身
    serializeContext["constValue"] = this.get_constValue();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.ConstActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_ConstActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.ConstActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.ConstActivity, this, 'execute', [context]);

    //将constValue放到activity的result之中
    var constValue = this.get_constValue();

    context.set_result(constValue);

    //执行完毕，结束activity
    $jwf.endActivity(context);
}

jsWorkFlow.Activities.ConstActivity.prototype = {
    _constValue: null,
    dispose: jsWorkFlow_Activities_ConstActivity$dispose,
    //property
    get_constValue: jsWorkFlow_Activities_ConstActivity$get_constValue,
    set_constValue: jsWorkFlow_Activities_ConstActivity$set_constValue,
    //method
    loadSerializeContext: jsWorkFlow_Activities_ConstActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_ConstActivity$saveSerializeContext,
    execute: jsWorkFlow_Activities_ConstActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.ConstActivity, 'jsWorkFlow.Activities.ConstActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//CompareActivity
//
//TO 开发者：
//    一个执行 相等性判断 操作的activity。
//    如果activity为null，其result被作为null处理。
//    内部先进行equals比较，如果不相等，lha > rha，返回1；如果lha < rha，返回-1；如果相等，返回0
//
jsWorkFlow.Activities.CompareActivity = function jsWorkFlow_Activities_CompareActivity(leftHandActivity, rightHandActivity, strict) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.CompareActivity, this);
    this.set_leftHandActivity(leftHandActivity);
    this.set_rightHandActivity(rightHandActivity);
    this.set_strict(strict);

    this._doExecLhaCompleteHandler = jso.createDelegate(this, this.doExecLhaCompleteHandler);
    this._doExecRhaCompleteHandler = jso.createDelegate(this, this.doExecRhaCompleteHandler);
};

function jsWorkFlow_Activities_CompareActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.CompareActivity, this, 'dispose');
}


function jsWorkFlow_Activities_CompareActivity$get_leftHandActivity() {
    return this._leftHandActivity;
}

function jsWorkFlow_Activities_CompareActivity$set_leftHandActivity(value) {
    this._leftHandActivity = value;
}

function jsWorkFlow_Activities_CompareActivity$get_rightHandActivity() {
    return this._rightHandActivity;
}

function jsWorkFlow_Activities_CompareActivity$set_rightHandActivity(value) {
    this._rightHandActivity = value;
}

function jsWorkFlow_Activities_CompareActivity$get_strict() {
    return this._strict;
}

function jsWorkFlow_Activities_CompareActivity$set_strict(value) {
    this._strict = value;
}

//activity的恢复
function jsWorkFlow_Activities_CompareActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity loadSerializeContext!");


    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.CompareActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.CompareActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var leftHandActivitySC = serializeContext['leftHandActivity'];
    var leftHandActivity = $jwf.loadActivity(leftHandActivitySC);
    this.set_leftHandActivity(leftHandActivity);

    var rightHandActivitySC = serializeContext['rightHandActivity'];
    var rightHandActivity = $jwf.loadActivity(rightHandActivitySC);
    this.set_rightHandActivity(rightHandActivity);

    var strict = serializeContext['strict'];
    this.set_strict(strict);
}

//activity的序列化
function jsWorkFlow_Activities_CompareActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.CompareActivity';

    //保存自身
    serializeContext['leftHandActivity'] = $jwf.saveActivity(this.get_leftHandActivity());
    serializeContext['rightHandActivity'] = $jwf.saveActivity(this.get_rightHandActivity());
    serializeContext['strict'] = this.get_strict();

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.CompareActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_CompareActivity$doExecLha(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity doExecLha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var lhaActivity = this._leftHandActivity;

    if (!lhaActivity) {
        this.doExecRha(context, null);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, lhaActivity, context);

    activityExecutor.add_postComplete(this._doExecLhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_CompareActivity$doExecLhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity doExecLhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //获取activity的最后的执行结果，作为doExecRha的输入
    var lhaResult = context.get_result();

    //执行Rha
    this.doExecRha(parentContext, lhaResult);

    return;
}

function jsWorkFlow_Activities_CompareActivity$doExecCompare(lhaResult, rhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity doExecCompare!");

    var strict = this._strict;
    var isEquals = (lhaResult == rhaResult);
    var result = 0;

    if (strict) {
        isEquals = (lhaResult === rhaResult);
    }

    if (!isEquals) {
        if (lhaResult > rhaResult) {
            result = 1;
        }
        else {
            result = -1;
        }
    }

    return result;
}

function jsWorkFlow_Activities_CompareActivity$doExecRha(context, lhaResult) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity doExecRha!");

    //开始执行lha
    //如果没有设置条件，认为为false，执行else分支
    var rhaActivity = this._rightHandActivity;

    if (!rhaActivity) {
        //结束activity的执行，并设置执行结果
        var result = this.doExecCompare(lhaResult, null);
        context.set_result(result);
        $jwf.endActivity(context);
        return;
    }

    //调度执行rha
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, rhaActivity, context);

    //将上下文的数据存放在activityExecutor之中
    activityExecutor.lhaResult = lhaResult;

    activityExecutor.add_postComplete(this._doExecRhaCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_CompareActivity$doExecRhaCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity doExecRhaCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var lhaResult = executor.lhaResult;

    var rhaResult = context.get_result();
    var result = this.doExecCompare(lhaResult, rhaResult);
    parentContext.set_result(result);

    //结束activity的执行
    $jwf.endActivity(parentContext);
    return;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_CompareActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.CompareActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.CompareActivity, this, 'execute', [context]);

    this.doExecLha(context);
}

jsWorkFlow.Activities.CompareActivity.prototype = {
    _leftHandActivity: null,
    _rightHandActivity: null,
    _doExecLhaCompleteHandler: null,
    _doExecRhaCompleteHandler: null,
    _strict: false,
    dispose: jsWorkFlow_Activities_CompareActivity$dispose,
    //property
    get_leftHandActivity: jsWorkFlow_Activities_CompareActivity$get_leftHandActivity,
    set_leftHandActivity: jsWorkFlow_Activities_CompareActivity$set_leftHandActivity,
    get_rightHandActivity: jsWorkFlow_Activities_CompareActivity$get_rightHandActivity,
    set_rightHandActivity: jsWorkFlow_Activities_CompareActivity$set_rightHandActivity,
    get_strict: jsWorkFlow_Activities_CompareActivity$get_strict,
    set_strict: jsWorkFlow_Activities_CompareActivity$set_strict,

    //method
    loadSerializeContext: jsWorkFlow_Activities_CompareActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_CompareActivity$saveSerializeContext,
    doExecLha: jsWorkFlow_Activities_CompareActivity$doExecLha,
    doExecLhaCompleteHandler: jsWorkFlow_Activities_CompareActivity$doExecLhaCompleteHandler,
    doExecCompare: jsWorkFlow_Activities_CompareActivity$doExecCompare,
    doExecRha: jsWorkFlow_Activities_CompareActivity$doExecRha,
    doExecRhaCompleteHandler: jsWorkFlow_Activities_CompareActivity$doExecRhaCompleteHandler,
    execute: jsWorkFlow_Activities_CompareActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.CompareActivity, 'jsWorkFlow.Activities.CompareActivity'),
    jsWorkFlow.Activity);

//////////////////////////////////////////////////////////////////////////////////////////
//IsEqualsActivity
//
//TO 开发者：
//    一个执行 相等性判断 操作的activity。
//    如果activity为null，其result被作为null处理。
//    进行equals比较，如果相等，返回true
//
jsWorkFlow.Activities.IsEqualsActivity = function jsWorkFlow_Activities_IsEqualsActivity(leftHandActivity, rightHandActivity, strict) {
    jso.initializeBase(jsWorkFlow.Activities.IsEqualsActivity, this, [leftHandActivity, rightHandActivity, strict]);
};

function jsWorkFlow_Activities_IsEqualsActivity$dispose() {
    jso.callBaseMethod(jsWorkFlow.Activities.IsEqualsActivity, this, 'dispose');
}



//activity的恢复
function jsWorkFlow_Activities_IsEqualsActivity$loadSerializeContext(serializeContext) {
    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.IsEqualsActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.IsEqualsActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
}

//activity的序列化
function jsWorkFlow_Activities_IsEqualsActivity$saveSerializeContext(serializeContext) {

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.IsEqualsActivity';

    //保存自身

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.IsEqualsActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_IsEqualsActivity$doExecCompare(lhaResult, rhaResult) {
    var strict = this._strict;
    var isEquals = (lhaResult == rhaResult);
    if (strict) {
        isEquals = (lhaResult === rhaResult);
    }

    return isEquals;
}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_IsEqualsActivity$execute(context) {
    jso.callBaseMethod(jsWorkFlow.Activities.IsEqualsActivity, this, 'execute', [context]);
}

jsWorkFlow.Activities.IsEqualsActivity.prototype = {
    dispose: jsWorkFlow_Activities_IsEqualsActivity$dispose,
    //property

    //method
    doExecCompare: jsWorkFlow_Activities_IsEqualsActivity$doExecCompare,
    execute: jsWorkFlow_Activities_IsEqualsActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.IsEqualsActivity, 'jsWorkFlow.Activities.IsEqualsActivity'), 
    jsWorkFlow.Activities.CompareActivity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//TODO;
//    Catch exception of activity
//////////////////////////////////////////////////////////////////////////////////////////
//TryCatchActivity
//
//TO 开发者：
//    TryCatchActivity，用于俘获运行期的异常，可以结合RaiseExceptionActivity使用。
//
jsWorkFlow.Activities.TryCatchActivity = function jsWorkFlow_Activities_TryCatchActivity(tryActivity, catchActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity create!");


    jso.initializeBase(jsWorkFlow.Activities.TryCatchActivity, this);

    this._tryActivity = tryActivity;
    this._catchActivity = catchActivity;

    this._doEvalTryCompleteHandler = jso.createDelegate(this, this.doEvalTryCompleteHandler);
    this._doEvalTryErrorHandler = jso.createDelegate(this, this.doEvalTryErrorHandler);
    this._doEvalTryErrorCompleteHandler = jso.createDelegate(this, this.doEvalTryErrorCompleteHandler);
    this._doEvalCatchCompleteHandler = jso.createDelegate(this, this.doEvalCatchCompleteHandler);

};

function jsWorkFlow_Activities_TryCatchActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.TryCatchActivity, this, 'dispose');
}


function jsWorkFlow_Activities_TryCatchActivity$get_tryActivity() {
    return this._tryActivity;
}

function jsWorkFlow_Activities_TryCatchActivity$set_tryActivity(value) {
    this._tryActivity = value;
}

function jsWorkFlow_Activities_TryCatchActivity$get_catchActivity() {
    return this._catchActivity;
}

function jsWorkFlow_Activities_TryCatchActivity$set_catchActivity(value) {
    this._catchActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_TryCatchActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.TryCatchActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.TryCatchActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var tryActivitySC = serializeContext["tryActivity"];
    var tryActivity = $jwf.loadActivity(tryActivitySC);
    this.set_tryActivity(tryActivity);

    var catchActivitySC = serializeContext["catchActivity"];
    var catchActivity = $jwf.loadActivity(catchActivitySC);
    this.set_catchActivity(catchActivity);
}

//activity的序列化
function jsWorkFlow_Activities_TryCatchActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.TryCatchActivity';

    //保存自身
    serializeContext['tryActivity'] = $jwf.saveActivity(this.get_tryActivity());
    serializeContext['catchActivity'] = $jwf.saveActivity(this.get_catchActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.TryCatchActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

//执行try block
function jsWorkFlow_Activities_TryCatchActivity$doEvalTry(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalTry!");

    //如果没有设置条件，认为为false，执行else分支
    var activity = this._tryActivity;

    if (!activity) {
        //没有try activity，结束执行
        $jwf.endActivity(context);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalTryCompleteHandler);
    activityExecutor.add_error(this._doEvalTryErrorHandler);
    activityExecutor.add_errorComplete(this._doEvalTryErrorCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_TryCatchActivity$doEvalTryCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalTryCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //运行到complete，表示正常运行没有出现错误，可以正常结束TryCatchActivity
    $jwf.endActivity(parentContext);
}

function jsWorkFlow_Activities_TryCatchActivity$doEvalTryErrorHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalTryErrorHandler!");

    //try activity
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    //tryCatchActivity
    var parentContext = executor.get_parentContext();

    //运行到error handler，表示发现runtime异常，需要执行catch activity

    //首先，通过设置executor的deliverError属性为false，阻断异常的向上传递
    executor.set_deliverError(false);

    this.doEvalCatch(context);


}


function jsWorkFlow_Activities_TryCatchActivity$doEvalTryErrorCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalTryErrorCompleteHandler!");

    //try activity
    var context = eventArgs.get_context();
    var executor = context.get_executor();
    //tryCatchActivity
    var parentContext = executor.get_parentContext();

    //恢复executor对异常的传递，errorComplete事件中已经完成了error的deliver，后面是对catch的错误的deliver
    executor.set_deliverError(true);

    //可以将error的handler去掉，这样如果在catch activity再发生异常，就可以继续向上传递了
    executor.remove_error(this._doEvalTryErrorHandler);
    executor.remove_errorComplete(this._doEvalTryErrorCompleteHandler);

}

function jsWorkFlow_Activities_TryCatchActivity$doEvalCatch(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalCatch!");

    var activity = this._catchActivity;

    if (!activity) {
        //如果没有catch activity，就简单的压制异常，然后结束parentContext
        $jwf.endActivity(parentContext);
    }

    //以错误的context作为parent，运行catch activity，这样可以获取到异常的错误码
    //这样，TryCatchActivity实际上是context的parentContext
    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalCatchCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_TryCatchActivity$doEvalCatchCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity doEvalCatchCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();
    var ppContext = parentContext.get_parentContext();

    //结束TryCatchActivity
    $jwf.endActivity(ppContext);

}


//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_TryCatchActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.TryCatchActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.TryCatchActivity, this, 'execute', [context]);

    this.doEvalTry(context);

}

jsWorkFlow.Activities.TryCatchActivity.prototype = {
    _tryActivity: null,
    _catchActivity: null,
    _doEvalTryCompleteHandler: null,
    _doEvalTryErrorHandler: null,
    _doEvalTryErrorCompleteHandler: null,
    _doEvalCatchCompleteHandler: null,
    dispose: jsWorkFlow_Activities_TryCatchActivity$dispose,
    //property
    get_tryActivity: jsWorkFlow_Activities_TryCatchActivity$get_tryActivity,
    set_tryActivity: jsWorkFlow_Activities_TryCatchActivity$set_tryActivity,
    get_catchActivity: jsWorkFlow_Activities_TryCatchActivity$get_catchActivity,
    set_catchActivity: jsWorkFlow_Activities_TryCatchActivity$set_catchActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_TryCatchActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_TryCatchActivity$saveSerializeContext,
    doEvalTry: jsWorkFlow_Activities_TryCatchActivity$doEvalTry,
    doEvalTryCompleteHandler: jsWorkFlow_Activities_TryCatchActivity$doEvalTryCompleteHandler,
    doEvalTryErrorHandler: jsWorkFlow_Activities_TryCatchActivity$doEvalTryErrorHandler,
    doEvalTryErrorCompleteHandler: jsWorkFlow_Activities_TryCatchActivity$doEvalTryErrorCompleteHandler,
    doEvalCatch: jsWorkFlow_Activities_TryCatchActivity$doEvalCatch,
    doEvalCatchCompleteHandler: jsWorkFlow_Activities_TryCatchActivity$doEvalCatchCompleteHandler,
    execute: jsWorkFlow_Activities_TryCatchActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.TryCatchActivity, 'jsWorkFlow.Activities.TryCatchActivity'), 
    jsWorkFlow.Activity);





//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');


//////////////////////////////////////////////////////////////////////////////////////////
//RaiseExceptionActivity
//
//TO 开发者：
//    一个RaiseExceptionActivity，用于发布一个异常，发布的异常可以通过TryCatchActivity拦截。
//
jsWorkFlow.Activities.RaiseExceptionActivity = function jsWorkFlow_Activities_RaiseExceptionActivity(exceptionActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity create!");


    jso.initializeBase(jsWorkFlow.Activities.RaiseExceptionActivity, this);

    this._exceptionActivity = exceptionActivity;
    this._doEvalExceptionActivityCompleteHandler = jso.createDelegate(this, this.doEvalExceptionActivityCompleteHandler);

};

function jsWorkFlow_Activities_RaiseExceptionActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.RaiseExceptionActivity, this, 'dispose');
}

function jsWorkFlow_Activities_RaiseExceptionActivity$get_exceptionActivity() {
    return this._exceptionActivity;
}

function jsWorkFlow_Activities_RaiseExceptionActivity$set_exceptionActivity(value) {
    this._exceptionActivity = value;
}

//activity的恢复
function jsWorkFlow_Activities_RaiseExceptionActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.RaiseExceptionActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.RaiseExceptionActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var exceptionActivitySC = serializeContext["exceptionActivity"];
    var exceptionActivity = $jwf.loadActivity(exceptionActivitySC);
    this.set_exceptionActivity(exceptionActivity);
}

//activity的序列化
function jsWorkFlow_Activities_RaiseExceptionActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.RaiseExceptionActivity';

    //保存自身
    var exceptionActivity = this.get_exceptionActivity();
    serializeContext["exceptionActivity"] = $jwf.saveActivity(exceptionActivity);

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.RaiseExceptionActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_RaiseExceptionActivity$doEvalExceptionActivity(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity doEvalExceptionActivity!");


    //如果没有设置条件，取值为null
    var activity = this._exceptionActivity;

    if (!activity) {
        //使用null来驱动doRaiseException执行
        this.doRaiseException(context, null, 0);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalExceptionActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();
}

function jsWorkFlow_Activities_SwitchActivity$doEvalExceptionActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity doEvalExceptionActivityCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var exceptionResult = context.get_result();

    log.debug("exceptionResult is:[" + exceptionResult + "]");

    //将condition传递给doEvalCaseCondition继续执行
    this.doRaiseException(parentContext, exceptionResult, 0);
}

function jsWorkFlow_Activities_SwitchActivity$doRaiseException(context, exceptionResult) {
    //结束context的执行，并引发异常
    //直接抛出异常，异常产生的源不一定会在正确的context上，因此需要使用$jwf.raiseError
    $jwf.raiseError(context, exceptionResult);

}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_RaiseExceptionActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.RaiseExceptionActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.RaiseExceptionActivity, this, 'execute', [context]);

    //获取异常信息，并抛出异常
    this.doEvalExceptionActivity(context);

    //会使当前的activity进入到error状态
}

jsWorkFlow.Activities.RaiseExceptionActivity.prototype = {
    _exceptionActivity: null,
    _doEvalExceptionActivityCompleteHandler: null,
    dispose: jsWorkFlow_Activities_RaiseExceptionActivity$dispose,
    //property
    get_exceptionActivity: jsWorkFlow_Activities_RaiseExceptionActivity$get_exceptionActivity,
    set_exceptionActivity: jsWorkFlow_Activities_RaiseExceptionActivity$set_exceptionActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_RaiseExceptionActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_RaiseExceptionActivity$saveSerializeContext,
    doEvalExceptionActivity: jsWorkFlow_Activities_RaiseExceptionActivity$doEvalExceptionActivity,
    doEvalExceptionActivityCompleteHandler: jsWorkFlow_Activities_SwitchActivity$doEvalExceptionActivityCompleteHandler,
    doRaiseException: jsWorkFlow_Activities_SwitchActivity$doRaiseException,
    execute: jsWorkFlow_Activities_RaiseExceptionActivity$execute
};

jso.registerClass(
    jso.setTypeName(jsWorkFlow.Activities.RaiseExceptionActivity, 'jsWorkFlow.Activities.RaiseExceptionActivity'), 
    jsWorkFlow.Activity);




//require namsepace
//jsWorkFlow.Activities namespace registed at core
jso.ns('jsWorkFlow.Activities', true);
var jsWorkFlow = jso.ns('jsWorkFlow');

//////////////////////////////////////////////////////////////////////////////////////////
//GetTypeNameActivity
//
//TO 开发者：
//    获取给出的activity的运行结果的数据类型。
//
jsWorkFlow.Activities.GetTypeNameActivity = function jsWorkFlow_Activities_GetTypeNameActivity(objActivity) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity create!");

    jso.initializeBase(jsWorkFlow.Activities.GetTypeNameActivity, this);

    this._objActivity = objActivity;
    this._doEvalObjActivityCompleteHandler = jso.createDelegate(this, this.doEvalObjActivityCompleteHandler);

};

function jsWorkFlow_Activities_GetTypeNameActivity$dispose() {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity dispose!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetTypeNameActivity, this, 'dispose');
}


function jsWorkFlow_Activities_GetTypeNameActivity$get_objActivity() {
    return this._objActivity;
}

function jsWorkFlow_Activities_GetTypeNameActivity$set_objActivity(value) {
    this._objActivity = value;
}


//activity的恢复
function jsWorkFlow_Activities_GetTypeNameActivity$loadSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity loadSerializeContext!");

    //检查类型 ===> 这是规范
    if (serializeContext['_@_activityType'] !== 'jsWorkFlow.Activities.GetTypeNameActivity') {
        throw jso.errorInvalidOperation("loadSerializeContext missmatch type!");
    }


    //恢复base
    var baseSerializeContext = serializeContext['_@_base'];

    jso.callBaseMethod(jsWorkFlow.Activities.GetTypeNameActivity, this, 'loadSerializeContext', [baseSerializeContext]);

    //恢复自身
    var objActivitySC = serializeContext["objActivity"];
    var objActivity = $jwf.loadActivity(objActivitySC);
    this.set_objActivity(objActivity);
}

//activity的序列化
function jsWorkFlow_Activities_GetTypeNameActivity$saveSerializeContext(serializeContext) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity saveSerializeContext!");

    //保存类型 ===> 这是规范
    serializeContext['_@_activityType'] = 'jsWorkFlow.Activities.GetTypeNameActivity';

    //保存自身
    serializeContext['objActivity'] = $jwf.saveActivity(this.get_objActivity());

    //保存base
    var baseSerializeContext = {};

    jso.callBaseMethod(jsWorkFlow.Activities.GetTypeNameActivity, this, 'saveSerializeContext', [baseSerializeContext]);

    serializeContext['_@_base'] = baseSerializeContext;
}

function jsWorkFlow_Activities_GetTypeNameActivity$doEvalObjActivity(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity doEvalObjActivity!");

    //如果没有设置条件，认为为false，执行else分支
    var activity = this._objActivity;

    if (!activity) {
        //如果没有定义activity，使用null替代
        this.doGetObjTypeName(context, null);
        return;
    }

    var application = context.get_application();
    var activityExecutor = new jsWorkFlow.ActivityExecutor(application, activity, context);

    activityExecutor.add_postComplete(this._doEvalObjActivityCompleteHandler);

    //OK, kick activityExecutor to run!
    activityExecutor.execute();

}

function jsWorkFlow_Activities_GetTypeNameActivity$doEvalObjActivityCompleteHandler(sender, eventArgs) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity doEvalObjActivityCompleteHandler!");

    var context = eventArgs.get_context();
    var executor = context.get_executor();
    var parentContext = executor.get_parentContext();

    //从context取执行结果
    var obj = context.get_result();

    //将condition传递给doExecuteBody继续执行
    this.doGetObjTypeName(parentContext, obj);
    
}

function jsWorkFlow_Activities_GetTypeNameActivity$doGetObjTypeName(context, obj) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity doGetObjTypeName!");

    //获取obj的typeName
    var typeName = "";

    if ((typeof (obj) !== "undefined") && (obj !== null)) {
        var objType = jso.getInstanceType(obj);

        typeName = jso.getTypeName(objType);
    }

    //见TypeName放到activity的执行结果中
    context.set_result(typeName);

    //执行完毕，结束activity
    $jwf.endActivity(context);

}

//activity的状态机的启动入口，自动驱动activity的状态机进入运行状态。
function jsWorkFlow_Activities_GetTypeNameActivity$execute(context) {
    var log = jwf$getLogger();
    log.debug("jsWorkFlow.Activities.GetTypeNameActivity execute!");

    jso.callBaseMethod(jsWorkFlow.Activities.GetTypeNameActivity, this, 'execute', [context]);

    this.doEvalObjActivity(context);

}

jsWorkFlow.Activities.GetTypeNameActivity.prototype = {
    _objActivity: null,
    _doEvalObjActivityCompleteHandler: null,
    dispose: jsWorkFlow_Activities_GetTypeNameActivity$dispose,
    //property
    get_objActivity: jsWorkFlow_Activities_GetTypeNameActivity$get_objActivity,
    set_objActivity: jsWorkFlow_Activities_GetTypeNameActivity$set_objActivity,
    //method
    loadSerializeContext: jsWorkFlow_Activities_GetTypeNameActivity$loadSerializeContext,
    saveSerializeContext: jsWorkFlow_Activities_GetTypeNameActivity$saveSerializeContext,
    doEvalObjActivity: jsWorkFlow_Activities_GetTypeNameActivity$doEvalObjActivity,
    doEvalObjActivityCompleteHandler: jsWorkFlow_Activities_GetTypeNameActivity$doEvalObjActivityCompleteHandler,
    doGetObjTypeName: jsWorkFlow_Activities_GetTypeNameActivity$doGetObjTypeName,
    execute: jsWorkFlow_Activities_GetTypeNameActivity$execute
};

jso.registerClass(jso.setTypeName(jsWorkFlow.Activities.GetTypeNameActivity, 'jsWorkFlow.Activities.GetTypeNameActivity'), jsWorkFlow.Activity);




}); // module: jsworkflowns

 return require("jsworkflowns");
})();